@@ -1,15 +1,13 @@
 /*
- * $Id: di.c,v 2.58 2009-11-29 17:42:57-08 bll Exp $
+ * $Id: di.c,v 2.63 2010-01-31 23:59:34-08 bll Exp $
  * $Source: /home/bll/DI/RCS/di.c,v $
  *
- * Copyright 1994-2009 Brad Lanam, Walnut Creek, CA
+ * Copyright 1994-2010 Brad Lanam, Walnut Creek, CA
  */
 
 /*
  * di.c
  *
- *   Copyright 1994-2009 Brad Lanam,  Walnut Creek, CA
- *
  *  Warning: Do not replace your system's 'df' command with this program.
  *           You will in all likelihood break your installation procedures.
  *
@@ -99,13 +97,19 @@
 #if _hdr_errno
 # include <errno.h>
 #endif
+#if _hdr_fcntl
+# include <fcntl.h>
+#endif
+#if _sys_file
+# include <sys/file.h>
+#endif
 #if _hdr_getopt
 # include <getopt.h>
 #endif
 #if _hdr_string
 # include <string.h>
 #endif
-#if _hdr_strings && ((! defined (_hdr_string)) || (_include_string))
+#if _hdr_strings && ((! _hdr_string) || (_include_string))
 # include <strings.h>
 #endif
 #if _hdr_memory
@@ -120,7 +124,7 @@
 #if _hdr_time
 # include <time.h>
 #endif
-#if _sys_time
+#if _sys_time && ((! _hdr_time) || (_include_time))
 # include <sys/time.h>
 #endif
 #if _sys_stat
@@ -141,17 +145,7 @@
 #endif
 
 #if _npt_getenv
-  extern char *getenv _((char *));
-#endif
-
-#if ! defined (_dcl_errno)
-  extern int     errno;
-#endif
-#if ! defined (_dcl_optind)
-  extern int optind;
-#endif
-#if ! defined (_dcl_optarg)
-  extern char *optarg;
+  extern char *getenv _((const char *));
 #endif
 
 #if defined(__cplusplus)
@@ -355,7 +349,7 @@ static dispTable_t dispTable [] =
   extern "C" {
 #endif
 
-static void addTotals           _((diDiskInfo_t *, diDiskInfo_t *, int));
+static void addTotals           _((const diDiskInfo_t *, diDiskInfo_t *, int));
 static void checkDiskInfo       _((diData_t *));
 static void getMaxFormatLengths _((diData_t *));
 static int  checkFileInfo       _((diData_t *, int, int, char *[]));
@@ -365,7 +359,7 @@ static void checkIncludeList    _((diDiskInfo_t *, iList_t *));
 static void checkZone           _((diDiskInfo_t *, zoneInfo_t *, int));
 #endif
 static void cleanup             _((diData_t *, char *));
-static int  diCompare           _((diOptions_t *, diDiskInfo_t *, unsigned int, unsigned int));
+static int  diCompare           _((const diOptions_t *, const diDiskInfo_t *, unsigned int, unsigned int));
 static int  findDispSize        _((double));
 static void getDiskSpecialInfo  _((diData_t *));
 static void getDiskStatInfo     _((diData_t *));
@@ -374,10 +368,10 @@ static void parseList           _((iList_t *, char *));
 static void preCheckDiskInfo    _((diData_t *));
 static void printDiskInfo       _((diData_t *));
 static void printInfo           _((diDiskInfo_t *, diOptions_t *));
-static void printBlocks         _((diOptions_t *, _fs_size_t, _fs_size_t, int));
+static void printBlocks         _((const diOptions_t *, _fs_size_t, _fs_size_t, int));
 static void printTitle          _((diOptions_t *));
-static void printPerc           _((_fs_size_t, _fs_size_t, char *));
-static void processArgs         _((int, char *[], diData_t *, char *));
+static void printPerc           _((_fs_size_t, _fs_size_t, const char *));
+static void processArgs         _((int, char * const [], diData_t *, char *));
 static void setDispBlockSize    _((char *, diOptions_t *));
 static void sortArray           _((diOptions_t *, diDiskInfo_t *, int));
 static void usage               _((void));
@@ -392,7 +386,7 @@ main (int argc, char *argv [])
 #else
 main (argc, argv)
     int                 argc;
-    char                *argv [];
+    char                * argv [];
 #endif
 {
     diData_t            diData;
@@ -488,6 +482,11 @@ main (argc, argv)
         char        *nargv [DI_MAX_ARGV];
 
         argvptr = strdup (ptr);
+        if (argvptr == (char *) NULL)
+        {
+            fprintf (stderr, "strdup failed in main() (1).  errno %d\n", errno);
+            exit (1);
+        }
         ooptind = optind;
         ooptarg = optarg;
         if (argvptr != (char *) NULL)
@@ -743,7 +742,7 @@ printDiskInfo (diData)
     if ((diopts->flags & DI_F_TOTAL) == DI_F_TOTAL)
     {
         memset ((char *) &totals, '\0', sizeof (diDiskInfo_t));
-        totals.blockSize = 8192;
+        totals.blockSize = 512;
         strncpy (totals.name, DI_GT("Total"), DI_NAME_LEN);
         totals.printFlag = DI_PRNT_OK;
     }
@@ -815,6 +814,8 @@ printDiskInfo (diData)
                 lastpoollen = strlen (lastpool);
                 inpool = FALSE;
               }
+            } else {
+              inpool = FALSE;
             }
 
             if (debug > 2)
@@ -1173,11 +1174,11 @@ printInfo (diskInfo, diopts)
 
 static void
 #if _proto_stdc
-printBlocks (diOptions_t *diopts, _fs_size_t blocks,
+printBlocks (const diOptions_t *diopts, _fs_size_t blocks,
                         _fs_size_t blockSize, int idx)
 #else
 printBlocks (diopts, blocks, blockSize, idx)
-    diOptions_t        *diopts;
+    const diOptions_t        *diopts;
     _fs_size_t          blocks;
     _fs_size_t          blockSize;
     int                 idx;
@@ -1187,7 +1188,7 @@ printBlocks (diopts, blocks, blockSize, idx)
     double          mult;
     double          temp;
     char            *suffix;
-    char            *format;
+    const char      *format;
 
 
     suffix = "";
@@ -1250,10 +1251,10 @@ findDispSize (siz)
 
 static void
 #if _proto_stdc
-addTotals (diDiskInfo_t *diskInfo, diDiskInfo_t *totals, int inpool)
+addTotals (const diDiskInfo_t *diskInfo, diDiskInfo_t *totals, int inpool)
 #else
 addTotals (diskInfo, totals, inpool)
-    diDiskInfo_t   *diskInfo;
+    const diDiskInfo_t   *diskInfo;
     diDiskInfo_t   *totals;
     int            inpool;
 #endif
@@ -1487,12 +1488,12 @@ printTitle (diopts)
 
 static void
 #if _proto_stdc
-printPerc (_fs_size_t used, _fs_size_t totAvail, char *format)
+printPerc (_fs_size_t used, _fs_size_t totAvail, const char *format)
 #else
 printPerc (used, totAvail, format)
     _fs_size_t      used;
     _fs_size_t      totAvail;
-    char            *format;
+    const char      *format;
 #endif
 {
     double      perc;
@@ -1545,7 +1546,19 @@ checkFileInfo (diData, optidx, argc, argv)
 
     for (i = optidx; i < argc; ++i)
     {
-        if (stat (argv [i], &statBuf) == 0)
+        int fd;
+        int src;
+
+        /* do this to automount devices.                    */
+        /* stat() will not necessarily cause an automount.  */
+        fd = open (argv[i], O_RDONLY | O_NOCTTY);
+        if (fd < 0)
+        {
+          src = stat (argv [i], &statBuf);
+        } else {
+          src = fstat (fd, &statBuf);
+        }
+        if (src == 0)
         {
             for (j = 0; j < diData->count; ++j)
             {
@@ -1631,21 +1644,21 @@ sortArray (diopts, data, count)
 
 static int
 #if _proto_stdc
-diCompare (diOptions_t *diopts, diDiskInfo_t *data,
+diCompare (const diOptions_t *diopts, const diDiskInfo_t *data,
            unsigned int idx1, unsigned int idx2)
 #else
 diCompare (diopts, data, idx1, idx2)
-    diOptions_t     *diopts;
-    diDiskInfo_t    *data;
+    const diOptions_t     *diopts;
+    const diDiskInfo_t    *data;
     unsigned int    idx1;
     unsigned int    idx2;
 #endif
 {
     int             rc;
     int             sortOrder;
-    char            *ptr;
-    diDiskInfo_t    *d1;
-    diDiskInfo_t    *d2;
+    const char            *ptr;
+    const diDiskInfo_t    *d1;
+    const diDiskInfo_t    *d2;
 
         /* reset sort order to the default start value */
     sortOrder = DI_SORT_ASCENDING;
@@ -2246,13 +2259,13 @@ usage ()
 static void
 #if _proto_stdc
 processArgs (int argc,
-             char *argv [],
+             char * const argv [],
              diData_t *diData,
              char *dbsstr)
 #else
 processArgs (argc, argv, diData, dbsstr)
     int             argc;
-    char            *argv [];
+    char            * const argv [];
     diData_t        *diData;
     char            *dbsstr;
 #endif
@@ -2445,7 +2458,7 @@ processArgs (argc, argv, diData, dbsstr)
             {
                 debug = atoi (optarg);
                 diopts->flags |= DI_F_DEBUG_HDR | DI_F_TOTAL;
-                diopts->flags &= ~ DI_F_NO_HEADER;
+                diopts->flags &= (__ulong) (~ DI_F_NO_HEADER);
                 diopts->width = 10;
                 diopts->inodeWidth = 10;
                 break;
@@ -2490,15 +2503,12 @@ parseList (list, str)
     unsigned int i;
     unsigned int len;
 
-    i = strlen (str);
-    dstr = (char *) malloc ((Size_t) i + 1);
+    dstr = strdup (str);
     if (dstr == (char *) NULL)
     {
-        fprintf (stderr, "malloc failed in parseList() (1).  errno %d\n", errno);
+        fprintf (stderr, "strdup failed in parseList() (1).  errno %d\n", errno);
         exit (1);
     }
-    memcpy (dstr, str, (Size_t) i);
-    dstr [i] = '\0';
 
     ptr = strtok (dstr, DI_LIST_SEP);
     count = 0;
