@@ -1,7 +1,7 @@
 /*
- * $Id: di.h,v 1.35 2009-11-29 17:39:50-08 bll Exp $
+ * $Id: di.h,v 1.40 2010-01-31 23:21:12-08 bll Exp $
  * $Source: /home/bll/DI/RCS/di.h,v $
- * Copyright 1994-2009 Brad Lanam, Walnut Creek, CA
+ * Copyright 1994-2010 Brad Lanam, Walnut Creek, CA
  */
 
 #ifndef __INC_DI_H_
@@ -12,6 +12,12 @@
 /*****************************************************/
 
 #include <stdio.h>
+#if _hdr_fcntl
+# include <fcntl.h>
+#endif
+#if _sys_file
+# include <sys/file.h>
+#endif
 #if _sys_types
 # include <sys/types.h>
 #endif
@@ -22,6 +28,10 @@
 # include <sys/param.h>     /* MAXPATHLEN */
 #endif
 
+#if ! defined (O_NOCTTY)
+# define O_NOCTTY 0
+#endif
+
 #if ! defined (MAXPATHLEN)
 # if defined (_POSIX_PATH_MAX)
 #  define MAXPATHLEN        _POSIX_PATH_MAX
@@ -45,7 +55,9 @@
 #  define DI_TYPE_LEN       FSTYPSZ
 # endif
 #endif
-#if _sys_mount                          /* NetBSD */
+#if _sys_mount && \
+    ! defined (_DI_INC_SYS_MOUNT)       /* NetBSD */
+# define _DI_INC_SYS_MOUNT 1
 # include <sys/mount.h>                 /* MFSNAMELEN */
 # if ! defined (DI_TYPE_LEN) && defined (MFSNAMELEN)
 #  define DI_TYPE_LEN       MFSNAMELEN
@@ -62,33 +74,48 @@
 # define DI_TYPE_LEN        65
 #endif
 
-#if ! defined (_lib_memcpy) && ! defined (memcpy)
-# if ! defined (_lib_bcopy)
-   error No memcpy/bcopy available.
+#if ! _lib_memcpy && ! defined (memcpy)
+# if ! _lib_bcopy
+   error No_memcpy/bcopy_available.
 # else
 #  define memcpy(dst, src, cnt)     (bcopy((src), (dst), (cnt)), dst)
 # endif
 #endif
 
-#if ! defined (_lib_memset) && ! defined (memset)
-# if ! defined (_lib_bzero)
-   error No memset/bzero available.
+#if ! _lib_memset && ! defined (memset)
+# if ! _lib_bzero
+   error No_memset/bzero_available.
 # else
 #  define memset(s,c,n)    (bzero ((s), (n)), s)
 # endif
 #endif
 
+#if ! _dcl_errno
+  extern int     errno;
+#endif
+#if ! _dcl_optind
+  extern int optind;
+#endif
+#if ! _dcl_optarg
+  extern char *optarg;
+#endif
+
 #define DI_NAME_LEN            MAXPATHLEN
 #define DI_SPEC_NAME_LEN       MAXPATHLEN
 #define DI_OPT_LEN             MAXPATHLEN
 #define DI_MNT_TIME_LEN        24
 
-#if _siz_long_long >= 8
-    typedef unsigned long long _fs_size_t;
-    typedef long long _s_fs_size_t;
-#else
+#if _siz_long_long == 0
     typedef unsigned long _fs_size_t;
     typedef long _s_fs_size_t;
+#else
+# if _siz_long_long >= 8
+    typedef unsigned long long _fs_size_t;
+    typedef long long _s_fs_size_t;
+# else
+    typedef unsigned long _fs_size_t;
+    typedef long _s_fs_size_t;
+# endif
 #endif
 
 typedef unsigned long __ulong;
@@ -142,6 +169,18 @@ typedef struct
 extern int  di_getDiskEntries       _((diDiskInfo_t **, int *));
  /* digetinfo.c */
 extern void di_getDiskInfo          _((diDiskInfo_t **, int *));
+ /* getopt.c */
+# if ! _lib_getopt || _npt_getopt
+extern int getopt                   _((char *, char * const [], const char *));
+# endif
+ /* strdup.c */
+# if ! _lib_strdup
+extern char *strdup                 _((const char *));
+# endif
+ /* strstr.c */
+# if ! _lib_strstr
+extern char *strstr                 _((const char *, const char *));
+# endif
  /* trimchar.c */
 extern void trimChar                _((char *, int));
  /* realloc.c */
@@ -151,12 +190,12 @@ extern void di_initDiskInfo _((diDiskInfo_t *));
 extern void di_saveBlockSizes _((diDiskInfo_t *, _fs_size_t, _fs_size_t, _fs_size_t, _fs_size_t));
 extern void di_saveInodeSizes _((diDiskInfo_t *, _fs_size_t, _fs_size_t, _fs_size_t));
 #if _lib_getmntent && \
-    ! defined (_lib_getmntinfo) && \
-    ! defined (_lib_getfsstat) && \
-    ! defined (_lib_getvfsstat) && \
-	! defined (_lib_mntctl) && \
-	! defined (_class_os__Volumes)
-extern char *chkMountOptions        _((char *, char *));
+    ! _lib_getmntinfo && \
+    ! _lib_getfsstat && \
+    ! _lib_getvfsstat && \
+	! _lib_mntctl && \
+	! _class_os__Volumes
+extern char *chkMountOptions        _((const char *, const char *));
 #endif
 extern void convertMountOptions     _((long, diDiskInfo_t *));
 extern void convertNFSMountOptions  _((long, long, long, diDiskInfo_t *));
@@ -164,8 +203,8 @@ extern void di_testRemoteDisk       _((diDiskInfo_t *));
 
 /* workaround for cygwin                                              */
 /* if we have a getopt header, there's probably a getopt lib function */
-# if ! defined (_lib_getopt) && ! defined (_hdr_getopt)
-extern int getopt _((int argc, char *argv [], char *optstring));
+# if ! _lib_getopt && ! _hdr_getopt
+extern int getopt _((int, char * const [], const char *));
 # endif
 
 # if defined(__cplusplus)
