@@ -1,8 +1,8 @@
 #!/bin/sh
 #
-# $Id: cflags.sh,v 1.40 2009-11-23 14:31:40-08 bll Exp $
+# $Id: cflags.sh,v 1.42 2010-01-31 11:30:05-08 bll Exp $
 # $Source: /home/bll/DI/features.base/RCS/cflags.sh,v $
-# Copyright 2001-2008 Brad Lanam, Walnut Creek, California USA
+# Copyright 2001-2010 Brad Lanam, Walnut Creek, California USA
 #
 
 bit64=""
@@ -160,13 +160,6 @@ case ${SYSTYPE} in
         # uname -m does not reflect actual architecture
         libs="-lroot -lbe $libs"
         ;;
-    syllable)
-        case ${TCC} in
-            cc|gcc)
-                CC=g++
-                ;;
-        esac
-        ;;
     CYGWIN*)
         ;;
     FreeBSD)
@@ -232,6 +225,9 @@ case ${SYSTYPE} in
         ;;
     NetBSD)
         ;;
+    OS/2)
+        ldflags="-Zexe"
+        ;;
     OSF1)
         ccflags="-std1 $ccflags"
         ;;
@@ -265,6 +261,13 @@ case ${SYSTYPE} in
                 ;;
         esac
         ;;
+    syllable)
+        case ${TCC} in
+            cc|gcc)
+                CC=g++
+                ;;
+        esac
+        ;;
     # unixware
     UNIX_SV)
         ;;
