@@ -1,11 +1,13 @@
 #!/bin/sh
 #
-# $Id: obj_ext.sh,v 1.8 2009-11-22 16:05:53-08 bll Exp $
+# $Id: obj_ext.sh,v 1.10 2010-01-31 11:30:08-08 bll Exp $
 # $Source: /home/bll/DI/features.base/RCS/obj_ext.sh,v $
-# Copyright 2001-2008 Brad Lanam, Walnut Creek, California, USA
+# Copyright 2001-2010 Brad Lanam, Walnut Creek, California, USA
 #
 
-if [ ! -f "object_ext" ]
+OBJEXT_FILE=object_ext
+
+if [ ! -f "${OBJEXT_FILE}" ]
 then
     cat > x$$.c << HERE
 #include <stdio.h>
@@ -31,17 +33,12 @@ HERE
          EXE_EXT=".exe"
     fi
     rm -f x$$.c x$$.o x$$.obj x$$ x$$.exe a.out > /dev/null 2>&1
-fi
-if [ -f "object_ext" ]
-then
-    eval `cat object_ext`
+
+    echo "OBJ_EXT=${OBJ_EXT};" > ${OBJEXT_FILE}
+    echo "EXE_EXT=${EXE_EXT};" >> ${OBJEXT_FILE}
+    echo "export OBJ_EXT;" >> ${OBJEXT_FILE}
+    echo "export EXE_EXT" >> ${OBJEXT_FILE}
 fi
 
-echo "OBJ_EXT=${OBJ_EXT};" > object_ext
-echo "EXE_EXT=${EXE_EXT}" >> object_ext
-echo "OBJ_EXT=${OBJ_EXT};"
-echo "EXE_EXT=${EXE_EXT};"
-echo "export OBJ_EXT;"
-echo "export EXE_EXT"
-
+cat ${OBJEXT_FILE}
 exit 0
