@@ -1,8 +1,8 @@
 #!/usr/bin/perl
 #
-# $Id: mkconfig.pl,v 1.38 2009-11-28 17:04:34-08 bll Exp $
+# $Id: mkconfig.pl,v 1.46 2010-01-31 22:42:49-08 bll Exp $
 #
-# Copyright 2006-2009 Brad Lanam Walnut Creek, CA USA
+# Copyright 2006-2010 Brad Lanam Walnut Creek, CA USA
 #
 
 # HP-UX doesn't have these installed.
@@ -10,8 +10,11 @@
 # use Config;
 require 5.005;
 
-my $LOG = "../mkconfig.log";
-my $TMP = "_tmp";
+my $CONFH = "config.h";
+my $LOG = "mkconfig.log";
+my $TMP = "_tmp_mkconfig";
+my $CACHEFILE = "mkconfig.cache";
+my $REQLIB = "reqlibs.txt";
 
 my $precc = <<'_HERE_';
 #if defined(__STDC__) || defined(__cplusplus) || defined(c_plusplus)
@@ -37,6 +40,165 @@ exitmkconfig
     exit 1;
 }
 
+sub
+printlabel
+{
+    my ($name, $label) = @_;
+    print LOGFH "## [$name] $label ... \n";
+    print STDOUT "$label ... ";
+}
+
+sub
+printyesno_val
+{
+    my ($name, $val, $tag) = @_;
+
+    if ($val ne "0")
+    {
+        print LOGFH "## [$name] $val $tag\n";
+        print STDOUT "$val $tag\n";
+    }
+    else
+    {
+        print LOGFH "## [$name] no $tag\n";
+        print STDOUT "no $tag\n";
+    }
+}
+
+sub
+printyesno
+{
+    my ($name, $val, $tag) = @_;
+
+    if ($val ne "0")
+    {
+        $val = "yes";
+    }
+    printyesno_val $name, $val, $tag;
+}
+
+sub
+savecache
+{
+    my ($r_clist, $r_config) = @_;
+
+    open (MKCC, ">$CACHEFILE");
+    foreach my $val (sort @{$r_clist->{'list'}})
+    {
+      print MKCC "di_cfg_${val}='" . $r_config->{$val} . "'\n";
+    }
+    my $vals = join (' ', @{$r_clist->{'list'}});
+    print MKCC "di_cfg_vars=' ${vals}'\n";
+    close (MKCC);
+}
+
+sub
+checkcache_val
+{
+  my ($name, $r_config) = @_;
+
+  my $val = $r_config->{$name};
+  my $rc = 1;
+  if (defined ($r_config->{$name}) && $val ne "" )
+  {
+    printyesno_val $name, $val, " (cached)";
+    $rc = 0;
+  }
+  return $rc;
+}
+
+sub
+checkcache
+{
+  my ($name, $r_config) = @_;
+
+  my $val = $r_config->{$name};
+  my $rc = 1;
+  if (defined ($r_config->{$name}) && $val ne "" )
+  {
+    printyesno $name, $val, " (cached)";
+    $rc = 0;
+  }
+  return $rc;
+}
+
+sub
+setlist
+{
+    my ($r_clist, $name) = @_;
+    my $r_hash = $r_clist->{'hash'};
+    if (! defined ($r_hash->{$name}))
+    {
+      push @{$r_clist->{'list'}}, $name;
+      $r_hash->{$name} = 1;
+    }
+}
+
+sub
+print_headers
+{
+    my ($r_a, $r_clist, $r_config) = @_;
+    my $txt;
+
+    $txt = '';
+
+    if ($r_a->{'incheaders'} eq 'all' ||
+        $r_a->{'incheaders'} eq 'std')
+    {
+        # always include these four if present ...
+        foreach my $val ('_hdr_stdio', '_hdr_stdlib', '_sys_types', '_sys_param')
+        {
+            if (defined ($r_config->{$val}) &&
+                 $r_config->{$val} ne '0')
+            {
+                $txt .= "#include <" . $r_config->{$val} . ">\n";
+            }
+        }
+    }
+
+    if ($r_a->{'incheaders'} eq 'all')
+    {
+        foreach my $val (@{$r_clist->{'list'}})
+        {
+            if ($val !~ m#^(_hdr_|_sys_)#o)
+            {
+                next;
+            }
+            if ($val eq '_hdr_stdio' ||
+                $val eq '_hdr_stdlib' ||
+                $val eq '_sys_types' ||
+                $val eq '_sys_param')
+            {
+                next;
+            }
+            if ($val eq '_hdr_malloc' &&
+                $r_config->{'_include_malloc'} eq '0')
+            {
+                next;
+            }
+            if ($val eq '_hdr_strings' &&
+                $r_config->{'_hdr_string'} ne '0' &&
+                $r_config->{'_include_string'} eq '0')
+            {
+                next;
+            }
+            if ($val eq '_sys_time' &&
+                $r_config->{'_hdr_time'} ne '0' &&
+                $r_config->{'_include_time'} eq '0')
+            {
+                next;
+            }
+            if ($r_config->{$val} ne '0')
+            {
+                $txt .= "#include <" . $r_config->{$val} . ">\n";
+            }
+        }
+        $txt .= "\n";
+    }
+
+    return $txt;
+}
+
 sub
 check_run
 {
@@ -81,17 +243,17 @@ check_link
     print CLFH $code;
     close CLFH;
 
-    system ("cat $name.c >> $LOG");
+    my $rc = system ("cat $name.c >> $LOG");
     if ($rc & 127) { exitmkconfig ($rc); }
 
     my $dlibs = '';
-    my $rc = _check_link ($name, {} );
+    $rc = _check_link ($name, {} );
     if ($rc != 0)
     {
       if ($otherlibs ne '')
       {
         my @olibs = split (/\s+/, $otherlibs);
-        $oliblist = '';
+        my $oliblist = '';
         foreach my $olib (@olibs)
         {
           $oliblist = $oliblist . ' ' . $olib;
@@ -172,13 +334,34 @@ check_compile
     return $rc;
 }
 
+sub
+do_check_compile
+{
+    my ($name, $code, $inc, $r_clist, $r_config) = @_;
+
+    my $rc = check_compile ($name, $code, $r_clist, $r_config,
+        { 'incheaders' => $inc, });
+    my $trc = 0;
+    if ($rc == 0)
+    {
+        $trc = 1;
+    }
+    printyesno $name, $trc;
+    setlist $r_clist, $name;
+    $r_config->{$name} = $trc;
+}
+
 sub
 check_header
 {
     my ($name, $file, $r_clist, $r_config, $r_a) = @_;
 
-    print LOGFH "## [$name] header: $file ... \n";
-    print STDERR "header: $file ... ";
+    printlabel $name, "header: $file";
+    if (checkcache ($name, $r_config) == 0)
+    {
+        return;
+    }
+
     my $r_rh = $r_a->{'reqhdr'} || [];
     my $code = '';
     foreach my $reqhdr (@$r_rh)
@@ -198,123 +381,35 @@ _HERE_
     if ($rc == 0)
     {
         $val = $file;
-        print LOGFH "## [$name] yes\n";
-        print STDERR "yes\n";
     }
-    else
-    {
-        print LOGFH "## [$name] no\n";
-        print STDERR "no\n";
-    }
-    push @$r_clist, $name;
+    printyesno $name, $val;
+    setlist $r_clist, $name;
     $r_config->{$name} = $val;
 }
 
 sub
-print_headers
+check_constant
 {
-    my ($r_a, $r_clist, $r_config) = @_;
-    my $txt;
+    my ($name, $constant, $r_clist, $r_config, $r_a) = @_;
 
-    $txt = '';
-
-    if ($r_a->{'incheaders'} eq 'all' ||
-        $r_a->{'incheaders'} eq 'std')
+    printlabel $name, "constant: $constant";
+    if (checkcache ($name, $r_config) == 0)
     {
-        # always include these four if present ...
-        foreach my $val ('_hdr_stdio', '_hdr_stdlib', '_sys_types', '_sys_param')
-        {
-            if (defined ($r_config->{$val}) &&
-                 $r_config->{$val} ne '0')
-            {
-                $txt .= "#include <" . $r_config->{$val} . ">\n";
-            }
-        }
+        return;
     }
 
-    if ($r_a->{'incheaders'} eq 'all')
+    my $r_rh = $r_a->{'reqhdr'} || [];
+    my $code = '';
+    foreach my $reqhdr (@$r_rh)
     {
-        foreach my $val (@$r_clist)
-        {
-            if ($val !~ m#^(_hdr_|_sys_)#o)
-            {
-                next;
-            }
-            if ($val eq '_hdr_stdio' ||
-                $val eq '_hdr_stdlib' ||
-                $val eq '_sys_types' ||
-                $val eq '_sys_param')
-            {
-                next;
-            }
-            if ($val eq '_hdr_malloc' &&
-                $r_config->{'_include_malloc'} eq '0')
-            {
-                next;
-            }
-            if ($val eq '_hdr_strings' &&
-                $r_config->{'_hdr_string'} ne '0' &&
-                $r_config->{'_include_string'} eq '0')
-            {
-                next;
-            }
-            if ($r_config->{$val} ne '0')
-            {
-                $txt .= "#include <" . $r_config->{$val} . ">\n";
-            }
-        }
-        $txt .= "\n";
-    }
-
-    return $txt;
-}
-
-sub
-check_class
-{
-    my ($name, $class, $r_clist, $r_config, $r_a) = @_;
-
-    push @$r_clist, $name;
-    $r_config->{$name} = 0;
-    my $code = <<"_HERE_";
-main () { $class testclass; }
+        $code .= <<"_HERE_";
+#include <$reqhdr>
 _HERE_
-    my $val = $r_a->{'otherlibs'} || '';
-
-    if ($val ne '')
-    {
-        print LOGFH "## [$name] class: $class [$val] ... \n";
-        print STDERR "class: $class [$val] ... ";
-    }
-    else
-    {
-        print LOGFH "## [$name] class: $class ... \n";
-        print STDERR "class: $class ... ";
-    }
-
-    my %a = (
-         'incheaders' => 'all',
-         'otherlibs' => $val,
-         );
-    my $rc = check_link ($name, $code, $r_clist, $r_config, \%a);
-    if ($rc == 0)
-    {
-      $r_config->{$name} = 1;
-      print LOGFH "## [$name] yes";
-      print STDERR "yes";
-      if ($a{'dlibs'} ne '')
-      {
-          print LOGFH " with $a{'dlibs'}";
-          print STDERR " with $a{'dlibs'}";
-      }
-      print LOGFH "\n";
-      print STDERR "\n";
-    }
-    else
-    {
-      print LOGFH "## [$name] no\n";
-      print STDERR "no\n";
     }
+    $code .= <<"_HERE_";
+main () { if (${constant} == 0) { 1; } exit (0); }
+_HERE_
+    do_check_compile ($name, $code, 'all', $r_clist, $r_config);
 }
 
 # if the keyword is reserved, the compile will fail.
@@ -323,26 +418,24 @@ check_keyword
 {
     my ($name, $keyword, $r_clist, $r_config) = @_;
 
-    print LOGFH "## [$name] keyword: $keyword ... \n";
-    print STDERR "keyword: $keyword ... ";
+    printlabel $name, "keyword: $keyword";
+    if (checkcache ($name, $r_config) == 0)
+    {
+        return;
+    }
+
     $r_config->{$name} = 0;
     my $code = <<"_HERE_";
 main () { int ${keyword}; ${keyword} = 1; exit (0); }
 _HERE_
     my $rc = check_compile ($name, $code, $r_clist, $r_config,
         { 'incheaders' => 'std', });
-    push @$r_clist, $name;
+    setlist $r_clist, $name;
     if ($rc != 0)  # failure means it is reserved...
     {
         $r_config->{$name} = 1;
-        print LOGFH "## [$name] yes\n";
-        print STDERR "yes\n";
-    }
-    else
-    {
-        print LOGFH "## [$name] no\n";
-        print STDERR "no\n";
     }
+    printyesno $name, $r_config->{$name};
 }
 
 sub
@@ -350,8 +443,12 @@ check_proto
 {
     my ($name, $r_clist, $r_config) = @_;
 
-    print LOGFH "## [$name] supported: prototypes ... \n";
-    print STDERR "supported: prototypes ... ";
+    printlabel $name, "supported: prototypes";
+    if (checkcache ($name, $r_config) == 0)
+    {
+        return;
+    }
+
     my $code = <<"_HERE_";
 _BEGIN_EXTERNS_
 extern int foo (int, int);
@@ -360,19 +457,13 @@ int bar () { int rc; rc = foo (1,1); return 0; }
 _HERE_
     my $rc = check_compile ($name, $code, $r_clist, $r_config,
         { 'incheaders' => 'all', });
-    push @$r_clist, $name;
+    setlist $r_clist, $name;
     $r_config->{$name} = 0;
     if ($rc == 0)
     {
         $r_config->{$name} = 1;
-        print LOGFH "## [$name] yes\n";
-        print STDERR "yes\n";
-    }
-    else
-    {
-        print LOGFH "## [$name] no\n";
-        print STDERR "no\n";
     }
+    printyesno $name, $r_config->{$name};
 }
 
 sub
@@ -380,25 +471,23 @@ check_command
 {
     my ($name, $cmd, $r_clist, $r_config) = @_;
 
-    print LOGFH "## [$name] command $cmd ... \n";
-    print STDERR "command: $cmd ... ";
+    printlabel $name, "command: $cmd";
+    if (checkcache ($name, $r_config) == 0)
+    {
+        return;
+    }
 
-    push @$r_clist, $name;
+    setlist $r_clist, $name;
     $r_config->{$name} = 0;
     foreach my $p (split /[;:]/o, $ENV{'PATH'})
     {
-        if (-x "$p/$cmd" && $r_config->{$name} == 0)
+        if (-x "$p/$cmd")
         {
-            $r_config->{$name} = 1;
-            print LOGFH "## [$name] yes\n";
-            print STDERR "yes\n";
+            $r_config->{$name} = "$p/$cmd";
+            last;
         }
     }
-    if ($r_config->{$name} == 0)
-    {
-        print LOGFH "## [$name] no\n";
-        print STDERR "no\n";
-    }
+    printyesno $name, $r_config->{$name};
 }
 
 # malloc.h conflicts w/string.h on some systems.
@@ -407,33 +496,24 @@ check_include_malloc
 {
     my ($name, $r_clist, $r_config) = @_;
 
-    push @$r_clist, $name;
-    $r_config->{$name} = 0;
-    if ($r_config->{'_hdr_malloc'} ne '0')
+    if (defined ($r_config->{'_hdr_malloc'}) &&
+        $r_config->{'_hdr_malloc'} ne '0' &&
+        defined ($r_config->{'_hdr_string'}) &&
+        $r_config->{'_hdr_string'} ne '0')
     {
-        print LOGFH "## [$name] header: include malloc.h ... \n";
-        print STDERR "header: include malloc.h ... ";
+        printlabel $name, "header: include malloc.h";
+        if (checkcache ($name, $r_config) == 0)
+        {
+            return;
+        }
 
         my $code = <<"_HERE_";
-main ()
-{
-    char *x;
-    x = (char *) malloc (20);
-}
+main () { char *x; x = (char *) malloc (20); }
 _HERE_
-        my $rc = check_compile ($name, $code, $r_clist, $r_config,
-                { 'incheaders' => 'all', });
-        if ($rc == 0)
-        {
-            $r_config->{$name} = 1;
-            print LOGFH "## [$name] yes\n";
-            print STDERR "yes\n";
-        }
-        else
-        {
-            print LOGFH "## [$name] no\n";
-            print STDERR "no\n";
-        }
+        do_check_compile ($name, $code, 'std', $r_clist, $r_config);
+    } else {
+        setlist $r_clist, $name;
+        $r_config->{$name} = 0;
     }
 }
 
@@ -442,35 +522,54 @@ check_include_string
 {
     my ($name, $r_clist, $r_config) = @_;
 
-    push @$r_clist, $name;
-    $r_config->{$name} = 0;
-    if ($r_config->{'_hdr_string'} ne '0' &&
+    if (defined ($r_config->{'_hdr_string'}) &&
+        $r_config->{'_hdr_string'} ne '0' &&
+        defined ($r_config->{'_hdr_strings'}) &&
         $r_config->{'_hdr_strings'} ne '0')
     {
-        print LOGFH "## [$name] header: include both string.h & strings.h ... \n";
-        print STDERR "header: include both string.h & strings.h ... ";
+        printlabel $name, "header: include both string.h & strings.h";
+        if (checkcache ($name, $r_config) == 0)
+        {
+            return;
+        }
 
         my $code = <<"_HERE_";
 #include <string.h>
 #include <strings.h>
-main ()
-{
-    char *x; x = "xyz"; strcat (x, "abc");
-}
+main () { char *x; x = "xyz"; strcat (x, "abc"); }
 _HERE_
-        my $rc = check_compile ($name, $code, $r_clist, $r_config,
-                { 'incheaders' => 'std', });
-        if ($rc == 0)
+        do_check_compile ($name, $code, 'std', $r_clist, $r_config);
+    } else {
+        setlist $r_clist, $name;
+        $r_config->{$name} = 0;
+    }
+}
+
+sub
+check_include_time
+{
+    my ($name, $r_clist, $r_config) = @_;
+
+    if (defined ($r_config->{'_hdr_time'}) &&
+        $r_config->{'_hdr_time'} ne '0' &&
+        defined ($r_config->{'_sys_time'}) &&
+        $r_config->{'_sys_time'} ne '0')
+    {
+        printlabel $name, "header: include both time.h & sys/time.h";
+        if (checkcache ($name, $r_config) == 0)
         {
-            $r_config->{$name} = 1;
-            print LOGFH "## [$name] yes\n";
-            print STDERR "yes\n";
-        }
-        else
-        {
-            print LOGFH "## [$name] no\n";
-            print STDERR "no\n";
+            return;
         }
+
+        my $code = <<"_HERE_";
+#include <time.h>
+#include <sys/time.h>
+main () { struct tm x; }
+_HERE_
+        do_check_compile ($name, $code, 'std', $r_clist, $r_config);
+    } else {
+        setlist $r_clist, $name;
+        $r_config->{$name} = 0;
     }
 }
 
@@ -479,29 +578,19 @@ check_npt
 {
     my ($name, $proto, $r_clist, $r_config) = @_;
 
-    print LOGFH "## [$name] need prototype: $proto ... \n";
-    print STDERR "need prototype: $proto ... ";
-    push @$r_clist, $name;
-    $r_config->{$name} = 0;
+    printlabel $name, "need prototype: $proto";
+    if (checkcache ($name, $r_config) == 0)
+    {
+        return;
+    }
+
     my $code = <<"_HERE_";
 _BEGIN_EXTERNS_
 struct _TEST_struct { int _TEST_member; };
 extern struct _TEST_struct* $proto _ARG_((struct _TEST_struct*));
 _END_EXTERNS_
 _HERE_
-    my $rc = check_compile ($name, $code, $r_clist, $r_config,
-            { 'incheaders' => 'all', });
-    if ($rc == 0)
-    {
-        $r_config->{$name} = 1;
-        print LOGFH "## [$name] yes\n";
-        print STDERR "yes\n";
-    }
-    else
-    {
-        print LOGFH "## [$name] no\n";
-        print STDERR "no\n";
-    }
+    do_check_compile ($name, $code, 'all', $r_clist, $r_config);
 }
 
 sub
@@ -509,29 +598,19 @@ check_type
 {
     my ($name, $type, $r_clist, $r_config) = @_;
 
-    print LOGFH "## [$name] type: $type ... \n";
-    print STDERR "type: $type ... ";
-    push @$r_clist, $name;
-    $r_config->{$name} = 0;
+    printlabel $name, "type: $type";
+    if (checkcache ($name, $r_config) == 0)
+    {
+        return;
+    }
+
     my $code = <<"_HERE_";
 struct xxx { $type mem; };
 static struct xxx v;
 struct xxx* f() { return &v; }
 main () { struct xxx *tmp; tmp = f(); exit (0); }
 _HERE_
-    my $rc = check_compile ($name, $code, $r_clist, $r_config,
-            { 'incheaders' => 'all', });
-    if ($rc == 0)
-    {
-        $r_config->{$name} = 1;
-        print LOGFH "## [$name] yes\n";
-        print STDERR "yes\n";
-    }
-    else
-    {
-        print LOGFH "## [$name] no\n";
-        print STDERR "no\n";
-    }
+    do_check_compile ($name, $code, 'all', $r_clist, $r_config);
 }
 
 sub
@@ -539,54 +618,89 @@ check_lib
 {
     my ($name, $func, $r_clist, $r_config, $r_a) = @_;
 
-    push @$r_clist, $name;
-    $r_config->{$name} = 0;
-    my $code = <<"_HERE_";
-typedef int (*_TEST_fun_)();
-#ifdef _TRY_extern_
-_BEGIN_EXTERNS_
-extern int $func();
-_END_EXTERNS_
-#endif
-static _TEST_fun_ i=(_TEST_fun_) $func;
-main () {  return (i==0); }
-_HERE_
+    setlist $r_clist, $name;
     my $val = $r_a->{'otherlibs'} || '';
 
     if ($val ne '')
     {
-        print LOGFH "## [$name] function: $func [$val] ... \n";
-        print STDERR "function: $func [$val] ... ";
+        printlabel $name, "function: $func [$val]";
     }
     else
     {
-        print LOGFH "## [$name] function: $func ... \n";
-        print STDERR "function: $func ... ";
+        printlabel $name, "function: $func";
+        if (checkcache ($name, $r_config) == 0)
+        {
+            return;
+        }
     }
 
+    $r_config->{$name} = 0;
+    # unfortunately, this does not work if the function
+    # is not declared.
+    my $code = <<"_HERE_";
+typedef int (*_TEST_fun_)();
+static _TEST_fun_ i=(_TEST_fun_) $func;
+main () {  i(); return (i==0); }
+_HERE_
+
     my %a = (
          'incheaders' => 'all',
          'otherlibs' => $val,
          );
     my $rc = check_link ($name, $code, $r_clist, $r_config, \%a);
+    my $tag = '';
     if ($rc == 0)
     {
       $r_config->{$name} = 1;
-      print LOGFH "## [$name] yes";
-      print STDERR "yes";
       if ($a{'dlibs'} ne '')
       {
-          print LOGFH " with $a{'dlibs'}";
-          print STDERR " with $a{'dlibs'}";
+          $tag = " with $a{'dlibs'}";
       }
-      print LOGFH "\n";
-      print STDERR "\n";
+    }
+    printyesno $name, $r_config->{$name}, $tag;
+}
+
+sub
+check_class
+{
+    my ($name, $class, $r_clist, $r_config, $r_a) = @_;
+
+    setlist $r_clist, $name;
+    my $val = $r_a->{'otherlibs'} || '';
+
+    if ($val ne '')
+    {
+        printlabel $name, "class: $class [$val]";
     }
     else
     {
-      print LOGFH "## [$name] no\n";
-      print STDERR "no\n";
+        printlabel $name, "class: $class";
+        if (checkcache ($name, $r_config) == 0)
+        {
+            return;
+        }
     }
+
+    $r_config->{$name} = 0;
+    my $code = <<"_HERE_";
+main () { $class testclass; }
+_HERE_
+
+    my %a = (
+         'incheaders' => 'all',
+         'otherlibs' => $val,
+         );
+    my $rc = check_link ($name, $code, $r_clist, $r_config, \%a);
+    my $tag = '';
+    if ($rc == 0)
+    {
+      $r_config->{$name} = 1;
+      if ($a{'dlibs'} ne '')
+      {
+          $tag = " with $a{'dlibs'}";
+      }
+    }
+    printyesno $name, $r_config->{$name}, $tag;
 }
 
 sub
@@ -594,9 +708,13 @@ check_setmntent_1arg
 {
     my ($name, $r_clist, $r_config) = @_;
 
-    print LOGFH "## [$name] setmntent(): 1 argument ... \n";
-    print STDERR "setmntent(): 1 argument ... ";
-    push @$r_clist, $name;
+    printlabel $name, "setmntent(): 1 argument";
+    if (checkcache ($name, $r_config) == 0)
+    {
+        return;
+    }
+
+    setlist $r_clist, $name;
     $r_config->{$name} = 0;
     my $code = <<"_HERE_";
 main () { setmntent ("/etc/mnttab"); }
@@ -606,14 +724,8 @@ _HERE_
     if ($rc == 0)
     {
         $r_config->{$name} = 1;
-        print LOGFH "## [$name] yes\n";
-        print STDERR "yes\n";
-    }
-    else
-    {
-        print LOGFH "## [$name] no\n";
-        print STDERR "no\n";
     }
+    printyesno $name, $r_config->{$name};
 }
 
 sub
@@ -621,9 +733,13 @@ check_setmntent_2arg
 {
     my ($name, $r_clist, $r_config) = @_;
 
-    print LOGFH "## [$name] setmntent(): 2 arguments ... \n";
-    print STDERR "setmntent(): 2 arguments ... ";
-    push @$r_clist, $name;
+    printlabel $name, "setmntent(): 2 arguments";
+    if (checkcache ($name, $r_config) == 0)
+    {
+        return;
+    }
+
+    setlist $r_clist, $name;
     $r_config->{$name} = 0;
     my $code = <<"_HERE_";
 main () { setmntent ("/etc/mnttab", "r"); }
@@ -633,14 +749,8 @@ _HERE_
     if ($rc == 0)
     {
         $r_config->{$name} = 1;
-        print LOGFH "## [$name] yes\n";
-        print STDERR "yes\n";
-    }
-    else
-    {
-        print LOGFH "## [$name] no\n";
-        print STDERR "no\n";
     }
+    printyesno $name, $r_config->{$name};
 }
 
 sub
@@ -648,9 +758,13 @@ check_statfs_2arg
 {
     my ($name, $r_clist, $r_config) = @_;
 
-    print LOGFH "## [$name] statfs(): 2 arguments ... \n";
-    print STDERR "statfs(): 2 arguments ... ";
-    push @$r_clist, $name;
+    printlabel $name, "statfs(): 2 arguments";
+    if (checkcache ($name, $r_config) == 0)
+    {
+        return;
+    }
+
+    setlist $r_clist, $name;
     $r_config->{$name} = 0;
     my $code = <<"_HERE_";
 main () {
@@ -663,14 +777,8 @@ _HERE_
     if ($rc == 0)
     {
         $r_config->{$name} = 1;
-        print LOGFH "## [$name] yes\n";
-        print STDERR "yes\n";
-    }
-    else
-    {
-        print LOGFH "## [$name] no\n";
-        print STDERR "no\n";
     }
+    printyesno $name, $r_config->{$name};
 }
 
 sub
@@ -678,9 +786,13 @@ check_statfs_3arg
 {
     my ($name, $r_clist, $r_config) = @_;
 
-    print LOGFH "## [$name] statfs(): 3 arguments ... \n";
-    print STDERR "statfs(): 3 arguments ... ";
-    push @$r_clist, $name;
+    printlabel $name, "statfs(): 3 arguments";
+    if (checkcache ($name, $r_config) == 0)
+    {
+        return;
+    }
+
+    setlist $r_clist, $name;
     $r_config->{$name} = 0;
     my $code = <<"_HERE_";
 main () {
@@ -693,14 +805,8 @@ _HERE_
     if ($rc == 0)
     {
         $r_config->{$name} = 1;
-        print LOGFH "## [$name] yes\n";
-        print STDERR "yes\n";
-    }
-    else
-    {
-        print LOGFH "## [$name] no\n";
-        print STDERR "no\n";
     }
+    printyesno $name, $r_config->{$name};
 }
 
 sub
@@ -708,9 +814,13 @@ check_statfs_4arg
 {
     my ($name, $r_clist, $r_config) = @_;
 
-    print LOGFH "## [$name] statfs(): 4 arguments ... \n";
-    print STDERR "statfs(): 4 arguments ... ";
-    push @$r_clist, $name;
+    printlabel $name, "statfs(): 4 arguments";
+    if (checkcache ($name, $r_config) == 0)
+    {
+        return;
+    }
+
+    setlist $r_clist, $name;
     $r_config->{$name} = 0;
     my $code = <<"_HERE_";
 main () {
@@ -723,14 +833,8 @@ _HERE_
     if ($rc == 0)
     {
         $r_config->{$name} = 1;
-        print LOGFH "## [$name] yes\n";
-        print STDERR "yes\n";
-    }
-    else
-    {
-        print LOGFH "## [$name] no\n";
-        print STDERR "no\n";
     }
+    printyesno $name, $r_config->{$name};
 }
 
 sub
@@ -738,9 +842,13 @@ check_size
 {
     my ($name, $type, $r_clist, $r_config) = @_;
 
-    print LOGFH "## [$name] sizeof: $type ... \n";
-    print STDERR "sizeof: $type ... ";
-    push @$r_clist, $name;
+    printlabel $name, "sizeof: $type";
+    if (checkcache ($name, $r_config) == 0)
+    {
+        return;
+    }
+
+    setlist $r_clist, $name;
     $r_config->{$name} = 0;
     my $code = <<"_HERE_";
 main () {
@@ -753,14 +861,8 @@ _HERE_
     if ($rc == 0)
     {
         $r_config->{$name} = $val;
-        print LOGFH "## [$name] $val\n";
-        print STDERR "$val\n";
-    }
-    else
-    {
-        print LOGFH "## [$name] no\n";
-        print STDERR "no\n";
     }
+    printyesno_val $name, $r_config->{$name};
 }
 
 sub
@@ -768,9 +870,13 @@ check_member
 {
     my ($name, $struct, $member, $r_clist, $r_config) = @_;
 
-    print LOGFH "## [$name] exists: $struct.$member ... \n";
-    print STDERR "exists: $struct.$member ... ";
-    push @$r_clist, $name;
+    printlabel $name, "exists: $struct.$member";
+    if (checkcache ($name, $r_config) == 0)
+    {
+        return;
+    }
+
+    setlist $r_clist, $name;
     $r_config->{$name} = 0;
     my $code = <<"_HERE_";
 main () { struct $struct s; int i; i = sizeof (s.$member); }
@@ -780,14 +886,8 @@ _HERE_
     if ($rc == 0)
     {
         $r_config->{$name} = 1;
-        print LOGFH "## [$name] yes\n";
-        print STDERR "yes\n";
-    }
-    else
-    {
-        print LOGFH "## [$name] no\n";
-        print STDERR "no\n";
     }
+    printyesno $name, $r_config->{$name};
 }
 
 sub
@@ -795,9 +895,13 @@ check_int_declare
 {
     my ($name, $function, $r_clist, $r_config) = @_;
 
-    print LOGFH "## [$name] declared: $function ... \n";
-    print STDERR "declared: $function ... ";
-    push @$r_clist, $name;
+    printlabel $name, "declared: $function";
+    if (checkcache ($name, $r_config) == 0)
+    {
+        return;
+    }
+
+    setlist $r_clist, $name;
     $r_config->{$name} = 0;
     my $code = <<"_HERE_";
     main () { int x; x = $function; }
@@ -807,14 +911,8 @@ _HERE_
     if ($rc == 0)
     {
         $r_config->{$name} = 1;
-        print LOGFH "## [$name] yes\n";
-        print STDERR "yes\n";
-    }
-    else
-    {
-        print LOGFH "## [$name] no\n";
-        print STDERR "no\n";
     }
+    printyesno $name, $r_config->{$name};
 }
 
 sub
@@ -822,9 +920,13 @@ check_ptr_declare
 {
     my ($name, $function, $r_clist, $r_config) = @_;
 
-    print LOGFH "## [$name] declared: $function ... \n";
-    print STDERR "declared: $function ... ";
-    push @$r_clist, $name;
+    printlabel $name, "declared: $function";
+    if (checkcache ($name, $r_config) == 0)
+    {
+        return;
+    }
+
+    setlist $r_clist, $name;
     $r_config->{$name} = 0;
     my $code = <<"_HERE_";
 main () { _VOID_ *x; x = $function; }
@@ -834,32 +936,54 @@ _HERE_
     if ($rc == 0)
     {
         $r_config->{$name} = 1;
-        print LOGFH "## [$name] yes\n";
-        print STDERR "yes\n";
-    }
-    else
-    {
-        print LOGFH "## [$name] no\n";
-        print STDERR "no\n";
     }
+    printyesno $name, $r_config->{$name};
 }
 
 
 sub
 create_config
 {
-    my (@clist, %config);
+    my ($configfile) = @_;
+    my (%clist, %config);
 
+    $clist{'list'} = ();
+    $clist{'hash'} = ();
     $config{'reqlibs'} = {};
 
-    open (CCOFH, ">../config.h");
+    open (CCOFH, ">$CONFH");
     print CCOFH <<'_HERE_';
-
 #ifndef __INC_CONFIG_H
 #define __INC_CONFIG_H 1
 
 _HERE_
 
+    if (-f $CACHEFILE)
+    {
+      open (MKCC, "<$CACHEFILE");
+      while (my $line = <MKCC>)
+      {
+        chomp $line;
+        if ($line =~ m/^di_cfg_(.*)='(.*)'/o)
+        {
+          my $name = $1;
+          my $val = $2;
+          if ($name eq 'vars') {
+            $val =~ s/^ *//;
+            @{$clist{'list'}} = split (/ +/, $val);
+            foreach my $var (@{$clist{'list'}})
+            {
+               $clist{'hash'}->{$var} = 1;
+            }
+          } else {
+            $config{$name} = $val;
+          }
+        }
+      }
+      close (MKCC);
+    }
+
+
     # FreeBSD has buggy headers, requires sys/param.h as a required include.
     # always check for these headers.
     my @headlist1 = (
@@ -871,16 +995,16 @@ _HERE_
 
     foreach my $r_arr (@headlist1)
     {
-        check_header ($$r_arr[0], $$r_arr[1], \@clist, \%config,
+        check_header ($$r_arr[0], $$r_arr[1], \%clist, \%config,
                 { 'reqhdr' => [], });
     }
-    check_keyword ('_key_void', 'void', \@clist, \%config);
-    check_keyword ('_key_const', 'const', \@clist, \%config);
-    check_proto ('_proto_stdc', \@clist, \%config);
+    check_keyword ('_key_void', 'void', \%clist, \%config);
+    check_keyword ('_key_const', 'const', \%clist, \%config);
+    check_proto ('_proto_stdc', \%clist, \%config);
 
-    if (! open (DATAIN, '../' . $ARGV[0]))
+    if (! open (DATAIN, "<../$configfile"))
     {
-        print STDERR "$ARGV[0]: $!\n";
+        print STDOUT "$configfile: $!\n";
         exit 1;
     }
 
@@ -897,8 +1021,9 @@ _HERE_
 
         if ($inheaders && $line !~ m#^(hdr|sys)#o)
         {
-            check_include_malloc ('_include_malloc', \@clist, \%config);
-            check_include_string ('_include_string', \@clist, \%config);
+            check_include_malloc ('_include_malloc', \%clist, \%config);
+            check_include_string ('_include_string', \%clist, \%config);
+            check_include_time ('_include_time', \%clist, \%config);
             $inheaders = 0;
         }
 
@@ -909,7 +1034,7 @@ _HERE_
         }
         elsif ($ininclude == 1)
         {
-            $line =~ s,\\\\,\\,go;
+            $line =~ s,\\(.),$1,g;
             $include .= $line . "\n";
         }
         elsif ($line =~ m#^include$#o)
@@ -939,7 +1064,22 @@ _HERE_
             if (! defined ($config{$nm}) ||
                 $config{$nm} eq '0')
             {
-                check_header ($nm, $hdr, \@clist, \%config,
+                check_header ($nm, $hdr, \%clist, \%config,
+                    { 'reqhdr' => \@oh, });
+            }
+        }
+        elsif ($line =~ m#^const\s+([^\s]+)\s*(.*)#o)
+        {
+            my $tnm = $1;
+            my $reqhdr = $2;
+            my $nm = "_const_" . lc $tnm;
+            $reqhdr =~ s/^\s*//o;
+            $reqhdr =~ s/\s*$//o;
+            my @oh = split (/\s+/, $reqhdr);
+            if (! defined ($config{$nm}) ||
+                $config{$nm} eq '0')
+            {
+                check_constant ($nm, $tnm, \%clist, \%config,
                     { 'reqhdr' => \@oh, });
             }
         }
@@ -950,22 +1090,22 @@ _HERE_
             if (! defined ($config{$nm}) ||
                 $config{$nm} eq '0')
             {
-                check_command ($nm, $cmd, \@clist, \%config);
+                check_command ($nm, $cmd, \%clist, \%config);
             }
         }
         elsif ($line =~ m#^npt\s+([^\s]*)\s*(.*)#o)
         {
             my $func = $1;
             my $req = $2;
-            if (! defined ($req) ||
-                $config{$req} ne '0')
+            my $nm = "_npt_" . lc $func;
+            if (! defined ($req) || $config{$req} ne '0')
             {
-                my $nm = "_npt_" . lc $func;
-                if (! defined ($config{$nm}) ||
-                    $config{$nm} eq '0')
+                if (! defined ($config{$nm}) || $config{$nm} eq '0')
                 {
-                    check_npt ($nm, $func, \@clist, \%config);
+                    check_npt ($nm, $func, \%clist, \%config);
                 }
+            } else {
+              $config{$nm} = 0;
             }
         }
         elsif ($line =~ m#^key\s+(.*)#o)
@@ -975,7 +1115,7 @@ _HERE_
             if (! defined ($config{$nm}) ||
                 $config{$nm} eq '0')
             {
-                check_keyword ($nm, $tnm, \@clist, \%config);
+                check_keyword ($nm, $tnm, \%clist, \%config);
             }
         }
         elsif ($line =~ m#^class\s+([^\s]+)\s*(.*)?#o)
@@ -987,7 +1127,7 @@ _HERE_
             if (! defined ($config{$nm}) ||
                 $config{$nm} eq '0')
             {
-                check_class ($nm, $class, \@clist, \%config,
+                check_class ($nm, $class, \%clist, \%config,
                        { 'otherlibs' => $libs, });
             }
         }
@@ -998,7 +1138,7 @@ _HERE_
             if (! defined ($config{$nm}) ||
                 $config{$nm} eq '0')
             {
-                check_type ($nm, $tnm, \@clist, \%config);
+                check_type ($nm, $tnm, \%clist, \%config);
             }
         }
         elsif ($line =~ m#^lib\s+([^\s]+)\s*(.*)?#o)
@@ -1009,18 +1149,18 @@ _HERE_
             if (! defined ($config{$nm}) ||
                 $config{$nm} eq '0')
             {
-                check_lib ($nm, $func, \@clist, \%config,
+                check_lib ($nm, $func, \%clist, \%config,
                        { 'otherlibs' => $libs, });
                 if ($func eq 'setmntent' && $config{$nm} ne '0')
                 {
-                    check_setmntent_1arg ('_setmntent_1arg', \@clist, \%config);
-                    check_setmntent_2arg ('_setmntent_2arg', \@clist, \%config);
+                    check_setmntent_1arg ('_setmntent_1arg', \%clist, \%config);
+                    check_setmntent_2arg ('_setmntent_2arg', \%clist, \%config);
                 }
                 if ($func eq 'statfs' && $config{$nm} ne '0')
                 {
-                    check_statfs_2arg ('_statfs_2arg', \@clist, \%config);
-                    check_statfs_3arg ('_statfs_3arg', \@clist, \%config);
-                    check_statfs_4arg ('_statfs_4arg', \@clist, \%config);
+                    check_statfs_2arg ('_statfs_2arg', \%clist, \%config);
+                    check_statfs_3arg ('_statfs_3arg', \%clist, \%config);
+                    check_statfs_4arg ('_statfs_4arg', \%clist, \%config);
                 }
             }
         }
@@ -1034,11 +1174,11 @@ _HERE_
             {
                 if ($type eq 'int')
                 {
-                    check_int_declare ($nm, $var, \@clist, \%config);
+                    check_int_declare ($nm, $var, \%clist, \%config);
                 }
                 elsif ($type eq 'ptr')
                 {
-                    check_ptr_declare ($nm, $var, \@clist, \%config);
+                    check_ptr_declare ($nm, $var, \%clist, \%config);
                 }
             }
         }
@@ -1050,7 +1190,7 @@ _HERE_
             if (! defined ($config{$nm}) ||
                 $config{$nm} eq '0')
             {
-                check_member ($nm, $struct, $member, \@clist, \%config);
+                check_member ($nm, $struct, $member, \%clist, \%config);
             }
         }
         elsif ($line =~ m#^size\s+(.*)#o)
@@ -1062,33 +1202,32 @@ _HERE_
             if (! defined ($config{$nm}) ||
                 $config{$nm} eq '0')
             {
-                check_size ($nm, $typ, \@clist, \%config);
+                check_size ($nm, $typ, \%clist, \%config);
             }
         }
         else
         {
-            print "unknown command: $line\n";
             print LOGFH "unknown command: $line\n";
+            print STDOUT "unknown command: $line\n";
         }
+        savecache (\%clist, \%config);
     }
 
-    foreach my $val (@clist)
+    foreach my $val (@{$clist{'list'}})
     {
-        if ($config{$val} eq '0')
-        {
-            print CCOFH "#undef  $val\n";
-        }
-        else
-        {
-            if ($val =~ m#^_siz_#o)
-            {
-                print CCOFH "#define $val $config{$val}\n";
-            }
-            else
-            {
-                print CCOFH "#define $val 1\n";
-            }
-        }
+      my $tval = 0;
+      if ($config{$val} ne "0")
+      {
+          $tval = 1;
+      }
+      if ($val =~ m#^_siz#o)
+      {
+        print CCOFH "#define $val $config{$val}\n";
+      }
+      else
+      {
+        print CCOFH "#define $val $tval\n";
+      }
     }
 
     # standard tail -- always needed; non specific
@@ -1122,27 +1261,100 @@ _HERE_
 _HERE_
     close CCOFH;
 
-    open (RLIBFH, ">../reqlibs.txt");
+    open (RLIBFH, ">$REQLIB");
 
     my $r_hash = $config{'reqlibs'};
     print RLIBFH join (' ', keys %$r_hash) . "\n";
     close RLIBFH;
+
+    savecache (\%clist, \%config);
 }
 
-##
-
-if (! defined ($ARGV[0]) ||
-    ! -f $ARGV[0])
+sub
+usage
 {
-    print STDERR "Usage: $0 <config-file>\n";
-    exit 1;
+  print STDOUT "Usage: $0 [-c <cache-file>] [-o <output-file>]";
+  print STDOUT "       [-l <log-file>] [-t <tmp-dir>] [-r <reqlib-file>]";
+  print STDOUT "       [-C] <config-file>";
+  print STDOUT "  -C : clear cache-file";
+  print STDOUT "<tmp-dir> must not exist.";
+  print STDOUT "defaults:";
+  print STDOUT "  <output-file>: config.h";
+  print STDOUT "  <cache-file> : mkconfig.cache";
+  print STDOUT "  <log-file>   : mkconfig.log";
+  print STDOUT "  <tmp-dir>    : _tmp_mkconfig";
+  print STDOUT "  <reqlib-file>: reqlibs.txt";
 }
 
+# main
+
+my $clearcache = 0;
+while ($#ARGV > 0)
+{
+  if ($ARGV[0] eq "-C")
+  {
+      shift @ARGV;
+      $clearcache = 1;
+  }
+  if ($ARGV[0] eq "-c")
+  {
+      shift @ARGV;
+      $CACHEFILE = $ARGV[0];
+      shift @ARGV;
+  }
+  if ($ARGV[0] eq "-o")
+  {
+      shift @ARGV;
+      $CONFH = $ARGV[0];
+      shift @ARGV;
+  }
+  if ($ARGV[0] eq "-l")
+  {
+      shift @ARGV;
+      $LOG = $ARGV[0];
+      shift @ARGV;
+  }
+  if ($ARGV[0] eq "-t")
+  {
+      shift @ARGV;
+      $TMP = $ARGV[0];
+      shift @ARGV;
+  }
+  if ($ARGV[0] eq "-r")
+  {
+      shift @ARGV;
+      $REQLIB = $ARGV[0];
+      shift @ARGV;
+  }
+}
+
+my $configfile = $ARGV[0];
+if (! defined ($configfile) || ! -f $configfile)
+{
+  usage;
+  exit 1;
+}
+if (-d $TMP && $TMP ne "_tmp_mkconfig")
+{
+  usage;
+  exit 1;
+}
+
+$LOG = "../$LOG";
+$CONFH = "../$CONFH";
+$REQLIB = "../$REQLIB";
+$CACHEFILE = "../$CACHEFILE";
+
 if (-d $TMP) { system ("rm -rf $TMP"); }
 mkdir $TMP, 0777;
 chdir $TMP;
 
-print "$0 using $ARGV[0]\n";
+if ($clearcache)
+{
+    unlink $CACHEFILE;
+}
+
+print STDOUT "$0 using $configfile\n";
 unlink $LOG;
 open (LOGFH, ">>$LOG");
 $ENV{'CFLAGS'} = $ENV{'CFLAGS'} . ' ' . $ENV{'CINCLUDES'};
@@ -1151,7 +1363,7 @@ print LOGFH "CFLAGS: $ENV{'CFLAGS'}\n";
 print LOGFH "LDFLAGS: $ENV{'LDFLAGS'}\n";
 print LOGFH "LIBS: $ENV{'LIBS'}\n";
 
-create_config;
+create_config $configfile;
 
 close LOGFH;
 
