@@ -1,18 +1,21 @@
 #!/bin/sh
 #
-# $Id: mkconfig.sh,v 1.10 2009-11-29 14:35:02-08 bll Exp $
+# $Id: mkconfig.sh,v 1.21 2010-01-31 23:17:27-08 bll Exp $
 #
-# Copyright 2009 Brad Lanam Walnut Creek, CA USA
+# Copyright 2009-2010 Brad Lanam Walnut Creek, CA USA
 #
 
-LOG="../mkconfig.log"
-CONFH="../config.h"
-REQLIB="../reqlibs.txt"
-TMP="_tmp"
-INC="include.txt"
-VARFILESUFFIX=".vars"
+LOG="mkconfig.log"
+CONFH="config.h"
+REQLIB="reqlibs.txt"
+TMP="_tmp_mkconfig"
+CACHEFILE="mkconfig.cache"
+datafile=""
+
+INC="include.txt"                   # temporary
 EN='-n'
 EC=''
+datachg=0
 
 precc='
 #if defined(__STDC__) || defined(__cplusplus) || defined(c_plusplus)
@@ -36,12 +39,27 @@ exitmkconfig () {
     exit 1
 }
 
+savedata () {
+    # And save the data for re-use.
+    # Some shells don't quote the values in the set
+    # command like bash does.  So we do it.
+    # Then we have to undo it for bash.
+    # And then there's: x='', which gets munged.
+    if [ $datachg -eq 1 ]; then
+      set | grep "^di_cfg" | \
+        sed "s/=/='/" | \
+        sed "s/$/'/" | \
+        sed "s/''/'/g" | \
+        sed "s/='$/=''/" \
+        > "${CACHEFILE}"
+      datachg=0
+    fi
+}
+
 cleardata () {
     prefix=$1
-    > "${prefix}${VARFILESUFFIX}"
     cmd="echo \${di_${prefix}_vars}"
-    for tval in `eval $cmd`
-    do
+    for tval in `eval $cmd`; do
         eval unset di_${prefix}_${tval}
     done
     eval "di_${prefix}_vars="
@@ -52,13 +70,18 @@ setdata () {
     sdname=$2
     sdval=$3
 
-    cmd="di_${prefix}_vars=\"\${di_${prefix}_vars} ${sdname}\""
+    if [ "$prefix" != "args" ]; then datachg=1; fi
+
+    cmd="echo \$di_${prefix}_vars | grep ${sdname} > /dev/null 2>&1"
     eval "$cmd"
+    rc=$?
+    # if already in the list of vars, don't add it again.
+    if [ $rc -ne 0 ]; then
+      cmd="di_${prefix}_vars=\"\${di_${prefix}_vars} ${sdname}\""
+      eval "$cmd"
+    fi
     cmd="di_${prefix}_${sdname}=\"${sdval}\""
     eval "$cmd"
-    set | grep "^di_${prefix}" | \
-        sed "s/=/='/" | sed "s/$/'/" | \
-        sed "s/''/'/g" > "${prefix}${VARFILESUFFIX}"
 }
 
 getdata () {
@@ -69,32 +92,103 @@ getdata () {
     echo $gdval
 }
 
-print_headers () {
+printlabel () {
+  tname="$1"
+  tlabel="$2"
 
+  echo "## [${tname}] ${tlabel} ... " >> $LOG
+  echo ${EN} "${tlabel} ... ${EC}"
+}
+
+printyesno_val () {
+  ynname=$1
+  ynval=$2
+  yntag="${3:-}"
+
+  if [ "$ynval" != "0" ]; then
+    echo "## [${ynname}] $ynval ${yntag}" >> $LOG
+    echo "$ynval ${yntag}"
+  else
+    echo "## [${ynname}] no ${yntag}" >> $LOG
+    echo "no ${yntag}"
+  fi
+}
+
+printyesno () {
+    ynname=$1
+    ynval=$2
+    yntag="${3:-}"
+
+    if [ "$ynval" != "0" ]; then
+      ynval="yes"
+    fi
+    printyesno_val $ynname $ynval "$yntag"
+}
+
+checkcache_val () {
+  tname=$1
+
+  tval=`getdata cfg $tname`
+  rc=1
+  if [ "$tval" != "" ]; then
+    printyesno_val $tname $tval " (cached)"
+    rc=0
+  fi
+  return $rc
+}
+
+checkcache () {
+  tname=$1
+
+  tval=`getdata cfg $tname`
+  rc=1
+  if [ "$tval" != "" ]; then
+    printyesno $tname $tval " (cached)"
+    rc=0
+  fi
+  return $rc
+}
+
+
+print_headers () {
     incheaders=`getdata args incheaders`
-    if [ "${incheaders}" = "all" -o "${incheaders}" = "std" ]
-    then
-        for tnm in '_hdr_stdio' '_hdr_stdlib' '_sys_types' '_sys_param'
-        do
+    if [ "${incheaders}" = "all" -o "${incheaders}" = "std" ]; then
+        for tnm in '_hdr_stdio' '_hdr_stdlib' '_sys_types' '_sys_param'; do
             tval=`getdata cfg ${tnm}`
-            if [ "${tval}" != "0" -a "${tval}" != "" ]
-            then
+            if [ "${tval}" != "0" -a "${tval}" != "" ]; then
                 echo "#include <${tval}>"
             fi
         done
     fi
 
-    if [ "${incheaders}" = "all" ]
-    then
-        for cfgvar in ${di_cfg_vars}
-        do
+    if [ "${incheaders}" = "all" ]; then
+        for cfgvar in ${di_cfg_vars}; do
+            hdval=`getdata cfg ${cfgvar}`
             case ${cfgvar} in
                 _hdr_stdio|_hdr_stdlib|_sys_types|_sys_param)
                     ;;
+                _hdr_malloc)
+                    imval=`getdata cfg '_include_malloc'`
+                    if [ "${imval}" != "0" ]; then
+                      echo "#include <${hdval}>"
+                    fi
+                    ;;
+                _hdr_strings)
+                    hsval=`getdata cfg '_hdr_string'`
+                    isval=`getdata cfg '_include_string'`
+                    if [ "${hsval}" = "0" -o "${isval}" != "0" ]; then
+                      echo "#include <${hdval}>"
+                    fi
+                    ;;
+                _sys_time)
+                    htval=`getdata cfg '_hdr_time'`
+                    itval=`getdata cfg '_include_time'`
+                    if [ "${htval}" = "0" -o "${itval}" != "0" ]; then
+                      echo "#include <${hdval}>"
+                    fi
+                    ;;
                 _hdr_*|_sys_*)
-                    hdval=`getdata cfg ${cfgvar}`
-                    if [ "${hdval}" != "0" -a "${hdval}" != "" ]
-                    then
+                    if [ "${hdval}" != "0" -a "${hdval}" != "" ]; then
                         echo "#include <${hdval}>"
                     fi
                     ;;
@@ -111,13 +205,11 @@ check_run () {
     rc=$?
     echo "##  run test: link: $rc" >> $LOG
     rval=0
-    if [ $rc -eq 0 ]
-    then
+    if [ $rc -eq 0 ]; then
         rval=`./${name}.exe`
         rc=$?
         echo "##  run test: run: $rc" >> $LOG
-        if [ $rc -lt 0 ]
-        then
+        if [ $rc -lt 0 ]; then
             exitmkconfig $rc
         fi
     fi
@@ -132,8 +224,7 @@ check_link () {
 
     ocounter=0
     clotherlibs=`getdata args otherlibs`
-    if [ "${clotherlibs}" != "" ]
-    then
+    if [ "${clotherlibs}" != "" ]; then
         set -- $clotherlibs
         ocount=$#
     else
@@ -151,16 +242,13 @@ check_link () {
     _check_link $name
     rc=$?
     echo "##      link test (none): $rc" >> $LOG
-    if [ $rc -ne 0 ]
-    then
-        while test $ocounter -lt $ocount
-        do
+    if [ $rc -ne 0 ]; then
+        while test $ocounter -lt $ocount; do
             ocounter=`expr $ocounter + 1`
             set -- $clotherlibs
             tcounter=0
             olibs=""
-            while test $tcounter -lt $ocounter
-            do
+            while test $tcounter -lt $ocounter; do
                 olibs="${olibs} $1"
                 shift
                 tcounter=`expr $tcounter + 1`
@@ -170,8 +258,7 @@ check_link () {
             _check_link $name
             rc=$?
             echo "##      link test (${olibs}): $rc" >> $LOG
-            if [ $rc -eq 0 ]
-            then
+            if [ $rc -eq 0 ]; then
                 break
             fi
         done
@@ -185,31 +272,26 @@ _check_link () {
 
     cmd="${CC} ${CFLAGS} "
     cflags=`getdata args cflags`
-    if [ "${cflags}" != "" ]
-    then
+    if [ "${cflags}" != "" ]; then
         cmd="${cmd} ${cflags} "
     fi
     cmd="${cmd} -o ${name}.exe ${name}.c "
     cmd="${cmd} ${LDFLAGS} ${LIBS} "
     _clotherlibs=`getdata args otherlibs`
-    if [ "${_clotherlibs}" != "" ]
-    then
+    if [ "${_clotherlibs}" != "" ]; then
         cmd="${cmd} ${_clotherlibs} "
     fi
     echo "##  _link test: $cmd" >> $LOG
     eval $cmd >> $LOG 2>&1
     rc=$?
-    if [ $rc -lt 0 ]
-    then
+    if [ $rc -lt 0 ]; then
         exitmkconfig $rc
     fi
     echo "##      _link test: $rc" >> $LOG
-    if [ $rc -eq 0 ]
-    then
-        if [ ! -x "${name}.exe" ]  # not executable.
-        then
-            rc=1
-        fi
+    if [ $rc -eq 0 ]; then
+      if [ ! -x "${name}.exe" ]; then  # not executable
+        rc=1
+      fi
     fi
     return $rc
 }
@@ -234,19 +316,54 @@ check_compile () {
 }
 
 
+do_check_compile () {
+    name="$1"
+    code="$2"
+    inc="$3"
+
+    cleardata args
+    setdata args incheaders ${inc}
+    check_compile "${name}" "${code}"
+    rc=$?
+    try="0"
+    if [ $rc -eq 0 ]; then
+        try="1"
+    fi
+    printyesno $name $try
+    setdata cfg "${name}" "${try}"
+}
+
+do_check_link () {
+    name="$1"
+    code="$2"
+    inc="$3"
+
+    cleardata args
+    setdata args incheaders ${inc}
+    setdata args otherlibs ""
+    check_link "${name}" "${code}" > /dev/null
+    rc=$?
+    trc=0
+    if [ $rc -eq 0 ]; then
+        trc=1
+    fi
+    printyesno $name $trc
+    setdata cfg "${name}" "${trc}"
+}
+
 check_header () {
     name=$1
     file=$2
 
-    echo "## [${name}] header: ${file} ... " >> $LOG
-    echo ${EN} "header: ${file} ... ${EC}"
+    printlabel $name "header: ${file}"
+    checkcache $name
+    if [ $rc -eq 0 ]; then return; fi
+
     reqhdr=`getdata args reqhdr`
     code=""
-    if [ "${reqhdr}" != "" ]
-    then
+    if [ "${reqhdr}" != "" ]; then
         set ${reqhdr}
-        while test $# -gt 0
-        do
+        while test $# -gt 0; do
             code="${code}
 #include <$1>
 "
@@ -263,97 +380,93 @@ main () { exit (0); }
     check_compile "${name}" "${code}"
     rc=$?
     val="0"
-    if [ $rc -eq 0 ]
-    then
+    if [ $rc -eq 0 ]; then
         val=${file}
-        echo "## [${name}] yes" >> $LOG
-        echo "yes"
-    else
-        echo "## [${name}] no" >> $LOG
-        echo "no"
     fi
+    printyesno $name $val
     setdata cfg "${name}" "${val}"
 }
 
+check_constant () {
+    name=$1
+    constant=$2
+
+    printlabel $name "constant: ${constant}"
+    checkcache $name
+    if [ $rc -eq 0 ]; then return; fi
+
+    reqhdr=`getdata args reqhdr`
+    code=""
+    if [ "${reqhdr}" != "" ]; then
+        set ${reqhdr}
+        while test $# -gt 0; do
+            code="${code}
+#include <$1>
+"
+            shift
+        done
+    fi
+    code="${code}
+main () { if (${constant} == 0) { 1; } exit (0); }
+"
+    do_check_compile "${name}" "${code}" all
+}
+
 check_keyword () {
     name=$1
     keyword=$2
 
-    echo "## [$name] keyword: $keyword ... " >> $LOG
-    echo ${EN} "keyword: $keyword ... ${EC}"
-    trc=0
+    printlabel $name "keyword: ${keyword}"
+    checkcache $name
+    if [ $rc -eq 0 ]; then return; fi
+
     code="main () { int ${keyword}; ${keyword} = 1; exit (0); }"
+
     cleardata args
     setdata args incheaders std
     check_compile "${name}" "${code}"
     rc=$?
-    if [ $rc -ne 0 ]  # failure means it is reserved...
-    then
-        trc=1
-        echo "## [$name] yes" >> $LOG
-        echo "yes"
-    else
-        echo "## [$name] no" >> $LOG
-        echo "no"
+    trc=0
+    if [ $rc -ne 0 ]; then  # failure means it is reserved...
+      trc=1
     fi
+    printyesno $name $trc
     setdata cfg "${name}" "${trc}"
 }
 
 check_proto () {
     name=$1
 
-    echo "## [$name] supported: prototypes ... " >> $LOG
-    echo ${EN} "supported: prototypes ... ${EC}"
+    printlabel $name "supported: prototypes"
+    checkcache $name
+    if [ $rc -eq 0 ]; then return; fi
+
     code='
 _BEGIN_EXTERNS_
 extern int foo (int, int);
 _END_EXTERNS_
 int bar () { int rc; rc = foo (1,1); return 0; }
 '
-    cleardata args
-    setdata args incheaders all
-    check_compile "${name}" "${code}"
-    rc=$?
-    trc=0
-    if [ $rc -eq 0 ]
-    then
-        trc=1
-        echo "## [$name] yes" >> $LOG
-        echo "yes"
-    else
-        echo "## [$name] no" >> $LOG
-        echo "no"
-    fi
-    setdata cfg "${name}" "${trc}"
+
+    do_check_compile "${name}" "${code}" all
 }
 
 check_type () {
     name=$1
     type=$2
 
-    echo "## [$name] type: $type ... " >> $LOG
-    echo ${EN} "type: $type ... ${EC}"
-    trc=0
+    printlabel $name "type: ${type}"
+    checkcache $name
+    if [ $rc -eq 0 ]; then return; fi
+
     code="
 struct xxx { ${type} mem; };
 static struct xxx v;
 struct xxx* f() { return &v; }
 main () { struct xxx *tmp; tmp = f(); exit (0); }
 "
-    cleardata args
-    setdata args incheaders all
-    check_compile "${name}" "${code}"
-    rc=$?
-    if [ $rc -eq 0 ]
-    then
-        trc=1
-        echo "## [$name] yes" >> $LOG
-        echo "yes"
-    else
-        echo "## [$name] no" >> $LOG
-        echo "no"
-    fi
-    setdata cfg "${name}" "${trc}"
+
+    do_check_compile "${name}" "${code}" all
 }
 
 check_member () {
@@ -361,162 +474,84 @@ check_member () {
     struct=$2
     member=$3
 
-    echo "## [$name] exists: $struct.$member ... " >> $LOG
-    echo ${EN} "exists: $struct.$member ... ${EC}"
-    trc=0
+    printlabel $name "exists: ${struct}.${member}"
+    checkcache $name
+    if [ $rc -eq 0 ]; then return; fi
+
     code="main () { struct ${struct} s; int i; i = sizeof (s.${member}); }"
-    cleardata args
-    setdata args incheaders all
-    check_compile "${name}" "${code}"
-    if [ $rc -eq 0 ]
-    then
-        trc=1
-        echo "## [$name] yes" >> $LOG
-        echo "yes"
-    else
-        echo "## [$name] no" >> $LOG
-        echo "no"
-    fi
-    setdata cfg "${name}" "${trc}"
+
+    do_check_compile "${name}" "${code}" all
 }
 
 check_setmntent_1arg () {
     name=$1
 
-    echo "## [$name] setmntent(): 1 argument ... " >> $LOG
-    echo ${EN} "setmntent(): 1 argument ... ${EC}"
-    trc=0
+    printlabel $name "setmntent(): 1 argument"
+    checkcache $name
+    if [ $rc -eq 0 ]; then return; fi
+
     code="main () { setmntent (\"/etc/mnttab\"); }"
-    cleardata args
-    setdata args incheaders all
-    setdata args otherlibs ""
-    check_link "${name}" "${code}" > /dev/null
-    rc=$?
-    if [ $rc -eq 0 ]
-    then
-        trc=1
-        echo "## [$name] yes" >> $LOG
-        echo "yes"
-    else
-        echo "## [$name] no" >> $LOG
-        echo "no"
-    fi
-    setdata cfg "${name}" "${trc}"
+
+    do_check_link "${name}" "${code}" all
 }
 
 check_setmntent_2arg () {
     name=$1
 
-    echo "## [$name] setmntent(): 2 arguments ... " >> $LOG
-    echo ${EN} "setmntent(): 2 arguments ... ${EC}"
+    printlabel $name "setmntent(): 2 arguments"
+    checkcache $name
+    if [ $rc -eq 0 ]; then return; fi
 
-    trc=0
     code="main () { setmntent (\"/etc/mnttab\", \"r\"); }"
-    cleardata args
-    setdata args incheaders all
-    setdata args otherlibs ""
-    check_link "${name}" "${code}" > /dev/null
-    rc=$?
-    if [ $rc -eq 0 ]
-    then
-        trc=1
-        echo "## [$name] yes" >> $LOG
-        echo "yes"
-    else
-        echo "## [$name] no" >> $LOG
-        echo "no"
-    fi
-    setdata cfg "${name}" "${trc}"
+    do_check_link "${name}" "${code}" all
 }
 
 check_statfs_2arg () {
     name=$1
 
-    echo "## [$name] statfs(): 2 arguments ... " >> $LOG
-    echo ${EN} "statfs(): 2 arguments ... ${EC}"
+    printlabel $name "statfs(): 2 arguments"
+    checkcache $name
+    if [ $rc -eq 0 ]; then return; fi
 
-    trc=0
     code="
 main () {
     struct statfs statBuf; char *name; name = \"/\";
     statfs (name, &statBuf);
 }
 "
-    cleardata args
-    setdata args incheaders all
-    setdata args otherlibs ""
-    check_link "${name}" "${code}" > /dev/null
-    rc=$?
-    if [ $rc -eq 0 ]
-    then
-        trc=1
-        echo "## [$name] yes" >> $LOG
-        echo "yes"
-    else
-        echo "## [$name] no" >> $LOG
-        echo "no"
-    fi
-    setdata cfg "${name}" "${trc}"
+    do_check_link "${name}" "${code}" all
 }
 
 check_statfs_3arg () {
     name=$1
 
-    echo "## [$name] statfs(): 3 arguments ... " >> $LOG
-    echo ${EN} "statfs(): 3 arguments ... ${EC}"
+    printlabel $name "statfs(): 3 arguments"
+    checkcache $name
+    if [ $rc -eq 0 ]; then return; fi
 
-    trc=0
     code="
 main () {
     struct statfs statBuf; char *name; name = \"/\";
     statfs (name, &statBuf, sizeof (statBuf));
 }
 "
-    cleardata args
-    setdata args incheaders all
-    setdata args otherlibs ""
-    check_link "${name}" "${code}" > /dev/null
-    rc=$?
-    if [ $rc -eq 0 ]
-    then
-        trc=1
-        echo "## [$name] yes" >> $LOG
-        echo "yes"
-    else
-        echo "## [$name] no" >> $LOG
-        echo "no"
-    fi
-    setdata cfg "${name}" "${trc}"
+    do_check_link "${name}" "${code}" all
 }
 
 check_statfs_4arg () {
     name=$1
 
-    echo "## [$name] statfs(): 4 arguments ... " >> $LOG
-    echo ${EN} "statfs(): 4 arguments ... ${EC}"
+    printlabel $name "statfs(): 4 arguments"
+    checkcache $name
+    if [ $rc -eq 0 ]; then return; fi
 
-    trc=0
     code="
 main () {
     struct statfs statBuf; char *name; name = \"/\";
     statfs (name, &statBuf, sizeof (statBuf), 0);
 }
 "
-    cleardata args
-    setdata args incheaders all
-    setdata args otherlibs ""
-    check_link "${name}" "${code}" > /dev/null
-    rc=$?
-    if [ $rc -eq 0 ]
-    then
-        trc=1
-        echo "## [$name] yes" >> $LOG
-        echo "yes"
-    else
-        echo "## [$name] no" >> $LOG
-        echo "no"
-    fi
-    setdata cfg "${name}" "${trc}"
+    do_check_link "${name}" "${code}" all
 }
 
 
@@ -524,20 +559,14 @@ check_size () {
     name=$1
     type=$2
 
-    echo "## [$name] sizeof: $type ... " >> $LOG
-    echo ${EN} "sizeof: $type ... ${EC}"
+    printlabel $name "sizeof: ${type}"
+    checkcache_val $name
+    if [ $rc -eq 0 ]; then return; fi
+
     code="main () { printf(\"%u\", sizeof(${type})); exit (0); }"
     val=0
     val=`check_run "${name}" "${code}"`
-    rc=$?
-    if [ $rc -eq 0 ]
-    then
-        echo "## [$name] $val" >> $LOG
-        echo "$val"
-    else
-        echo "## [$name] no" >> $LOG
-        echo "no"
-    fi
+    printyesno_val $name $val
     setdata cfg "${name}" "${val}"
 }
 
@@ -545,102 +574,60 @@ check_int_declare () {
     name=$1
     function=$2
 
-    echo "## [$name] declared: $function ... " >> $LOG
-    echo ${EN} "declared: $function ... ${EC}"
-    trc=0
+    printlabel $name "declared: ${function}"
+    checkcache $name
+    if [ $rc -eq 0 ]; then return; fi
+
     code="main () { int x; x = ${function}; }"
-    cleardata args
-    setdata args incheaders all
-    check_compile "${name}" "${code}"
-    rc=$?
-    if [ $rc -eq 0 ]
-    then
-        trc=1
-        echo "## [$name] yes" >> $LOG
-        echo "yes"
-    else
-        echo "## [$name] no" >> $LOG
-        echo "no"
-    fi
-    setdata cfg "${name}" "${trc}"
+    do_check_compile "${name}" "${code}" all
 }
 
 check_ptr_declare () {
     name=$1
     function=$2
 
-    echo "## [$name] declared: $function ... " >> $LOG
-    echo ${EN} "declared: $function ... ${EC}"
-    trc=0
+    printlabel $name "declared: ${function}"
+    checkcache $name
+    if [ $rc -eq 0 ]; then return; fi
+
     code="main () { _VOID_ *x; x = ${function}; }"
-    cleardata args
-    setdata args incheaders all
-    check_compile "${name}" "${code}"
-    rc=$?
-    if [ $rc -eq 0 ]
-    then
-        trc=1
-        echo "## [$name] yes" >> $LOG
-        echo "yes"
-    else
-        echo "## [$name] no" >> $LOG
-        echo "no"
-    fi
-    setdata cfg "${name}" "${trc}"
+    do_check_compile "${name}" "${code}" all
 }
 
 check_npt () {
     name=$1
     proto=$2
 
-    echo "## [$name] need prototype: $proto ... " >> $LOG
-    echo ${EN} "need prototype: $proto ... ${EC}"
-    trc=0
+    printlabel $name "need prototype: ${proto}"
+    checkcache $name
+    if [ $rc -eq 0 ]; then return; fi
+
     code="
 _BEGIN_EXTERNS_
 struct _TEST_struct { int _TEST_member; };
 extern struct _TEST_struct* ${proto} _ARG_((struct _TEST_struct*));
 _END_EXTERNS_
 "
-    cleardata args
-    setdata args incheaders all
-    check_compile "${name}" "${code}"
-    rc=$?
-    if [ $rc -eq 0 ]
-    then
-        trc=1
-        echo "## [$name] yes" >> $LOG
-        echo "yes"
-    else
-        echo "## [$name] no" >> $LOG
-        echo "no"
-    fi
-    setdata cfg "${name}" "${trc}"
+    do_check_compile "${name}" "${code}" all
 }
 
 check_command () {
     name=$1
     cmd=$2
 
-    echo "## [$name] command $cmd ... " >> $LOG
-    echo ${EN} "command: $cmd ... ${EC}"
+    printlabel $name "command: ${cmd}"
+    checkcache $name
+    if [ $rc -eq 0 ]; then return; fi
 
     trc=0
     pth="`echo ${PATH} | sed 's/[;:]/ /g'`"
-    for p in $pth
-    do
-        if [ $trc -eq  0 -a -x "$p/$cmd" ]
-        then
-            trc=1
-            echo "## [$name] yes" >> $LOG
-            echo "yes"
+    for p in $pth; do
+        if [ -x "$p/$cmd" ]; then
+            trc="$p/$cmd"
+            break
         fi
     done
-    if [ $trc -eq 0 ]
-    then
-        echo "## [$name] no" >> $LOG
-        echo "no"
-    fi
+    printyesno $name $trc
     setdata cfg "${name}" "${trc}"
 }
 
@@ -648,52 +635,41 @@ check_lib () {
     name=$1
     func=$2
 
-    trc=0
-    code="
-typedef int (*_TEST_fun_)();
-#ifdef _TRY_extern_
-_BEGIN_EXTERNS_
-extern int ${func}();
-_END_EXTERNS_
-#endif
-static _TEST_fun_ i=(_TEST_fun_) ${func};
-main () {  return (i==0); }
-"
     otherlibs=`getdata args otherlibs`
 
-    if [ "${otherlibs}" != "" ]
-    then
-        echo "## [$name] function: $func [$otherlibs] ... " >> $LOG
-        echo ${EN} "function: $func [$otherlibs] ... ${EC}"
+    if [ "${otherlibs}" != "" ]; then
+        printlabel $name "function: ${func} [${otherlibs}]"
     else
-        echo "## [$name] function: $func ... " >> $LOG
-        echo ${EN} "function: $func ... ${EC}"
+        printlabel $name "function: ${func}"
+        checkcache $name
+        if [ $rc -eq 0 ]; then return; fi
     fi
 
+    trc=0
+    # unfortunately, this does not work if the function
+    # is not declared.
+    code="
+typedef int (*_TEST_fun_)();
+static _TEST_fun_ i=(_TEST_fun_) ${func};
+main () {  i(); return (i==0); }
+"
+
     cleardata args
     setdata args incheaders all
     setdata args otherlibs "${otherlibs}"
     dlibs=`check_link "${name}" "${code}"`
     rc=$?
-    if [ $rc -eq 0 ]
-    then
+    if [ $rc -eq 0 ]; then
         trc=1
-        echo ${EN} "## [$name] yes${EC}" >> $LOG
-        echo ${EN} "yes${EC}"
-        if [ "$dlibs" != "" ]
-        then
-            echo ${EN} " with ${dlibs} ${EC}" >> $LOG
-            echo ${EN} " with ${dlibs} ${EC}"
-            reqlibs=`getdata data reqlibs`
-            reqlibs="${reqlibs} ${dlibs}"
-            setdata data reqlibs "${reqlibs}"
-        fi
-        echo "" >> $LOG
-        echo ""
-    else
-        echo "## [$name] no" >> $LOG
-        echo "no"
     fi
+    tag=""
+    if [ $rc -eq 0 -a "$dlibs" != "" ]; then
+      tag=" with ${dlibs}"
+      reqlibs=`getdata data reqlibs`
+      reqlibs="${reqlibs} ${dlibs}"
+      setdata data reqlibs "${reqlibs}"
+    fi
+    printyesno $name $trc "$tag"
     setdata cfg "${name}" "${trc}"
     return $trc
 }
@@ -706,13 +682,12 @@ check_class () {
     code=" main () { ${class} testclass; } "
     otherlibs=`getdata args otherlibs`
 
-    if [ "$otherlibs" != "" ]
-    then
-        echo "## [$name] class: $class [$otherlibs] ... " >> $LOG
-        echo ${EN} "class: $class [$otherlibs] ... ${EC}"
+    if [ "$otherlibs" != "" ]; then
+        printlabel $name "class: ${class} [${otherlibs}]"
     else
-        echo "## [$name] class: $class ... " >> $LOG
-        echo ${EN} "class: $class ... ${EC}"
+        printlabel $name "class: ${class}"
+        checkcache $name
+        if [ $rc -eq 0 ]; then return; fi
     fi
 
     cleardata args
@@ -720,25 +695,17 @@ check_class () {
     setdata args otherlibs "${otherlibs}"
     check_link "${name}" "${code}" > /dev/null
     rc=$?
-    if [ $rc -eq 0 ]
-    then
+    if [ $rc -eq 0 ]; then
         trc=1
-        echo ${EN} "## [$name] yes${EC}" >> $LOG
-        echo ${EN} "yes${EC}"
-        if [ "${dlibs}" != "" ]
-        then
-            echo ${EN} " with ${dlibs}${EC}" >> $LOG
-            echo ${EN} " with ${dlibs}${EC}"
-            reqlibs=`getdata data reqlibs`
-            reqlibs="${reqlibs} ${dlibs}"
-            setdata data reqlibs "${reqlibs}"
-        fi
-        echo "" >> $LOG
-        echo ""
-    else
-        echo "## [$name] no" >> $LOG
-        echo "no"
     fi
+    tag=""
+    if [ $rc -eq 0 -a "${dlibs}" != "" ]; then
+        tag=" with ${dlibs}"
+        reqlibs=`getdata data reqlibs`
+        reqlibs="${reqlibs} ${dlibs}"
+        setdata data reqlibs "${reqlibs}"
+    fi
+    printyesno $name $trc "$tag"
     setdata cfg "${name}" "${trc}"
 }
 
@@ -749,30 +716,19 @@ check_include_malloc () {
     trc=0
     _hdr_malloc=`getdata cfg _hdr_malloc`
     _hdr_string=`getdata cfg _hdr_string`
-    if [ "${_hdr_string}" = "string.h" -a "${_hdr_malloc}" = "malloc.h" ]
-    then
-        echo "## [$name] header: include malloc.h ... " >> $LOG
-        echo ${EN} "header: include malloc.h ... ${EC}"
+    if [ "${_hdr_string}" = "string.h" -a "${_hdr_malloc}" = "malloc.h" ]; then
+      printlabel $name "header: include malloc.h"
+      checkcache $name
+      if [ $rc -eq 0 ]; then return; fi
 
-        code="
+      code="
 #include <string.h>
 #include <malloc.h>
 main () { char *x; x = (char *) malloc (20); }"
-        cleardata args
-        setdata args incheaders std
-        check_compile "${name}" "${code}"
-        rc=$?
-        if [ $rc -eq 0 ]
-        then
-            trc=1
-            echo "## [$name] yes" >> $LOG
-            echo "yes"
-        else
-            echo "## [$name] no" >> $LOG
-            echo "no"
-        fi
+      do_check_compile "${name}" "${code}" std
+    else
+      setdata cfg "${name}" "${trc}"
     fi
-    setdata cfg "${name}" "${trc}"
 }
 
 check_include_string () {
@@ -781,47 +737,60 @@ check_include_string () {
     trc=0
     _hdr_string=`getdata cfg _hdr_string`
     _hdr_strings=`getdata cfg _hdr_strings`
-    if [ "${_hdr_string}" = "string.h" -a "${_hdr_strings}" = "strings.h" ]
-    then
-        echo "## [$name] header: include both string.h & strings.h ... " >> $LOG
-        echo ${EN} "header: include both string.h & strings.h ... ${EC}"
+    if [ "${_hdr_string}" = "string.h" -a "${_hdr_strings}" = "strings.h" ]; then
+      printlabel $name "header: include both string.h & strings.h"
+      checkcache $name
+      if [ $rc -eq 0 ]; then return; fi
 
-        code="#include <string.h>
+      code="#include <string.h>
 #include <strings.h>
 main () { char *x; x = \"xyz\"; strcat (x, \"abc\"); }
 "
-        cleardata args
-        setdata args incheaders std
-        check_compile "${name}" "${code}"
-        rc=$?
-        if [ $rc -eq 0 ]
-        then
-            trc=1
-            echo "## [$name] yes" >> $LOG
-            echo "yes"
-        else
-            echo "## [$name] no" >> $LOG
-            echo "no"
-        fi
+      do_check_compile "${name}" "${code}" std
+    else
+      setdata cfg "${name}" "${trc}"
+    fi
+}
+
+check_include_time () {
+    name=$1
+
+    trc=0
+    _hdr_time=`getdata cfg _hdr_time`
+    _sys_time=`getdata cfg _sys_time`
+    if [ "${_hdr_time}" = "time.h" -a "${_sys_time}" = "sys/time.h" ]; then
+      printlabel $name "header: include both time.h & sys/time.h"
+      checkcache $name
+      if [ $rc -eq 0 ]; then return; fi
+
+      code="#include <time.h>
+#include <sys/time.h>
+main () { struct tm x; }
+"
+      do_check_compile "${name}" "${code}" std
+    else
+      setdata cfg "${name}" "${trc}"
     fi
-    setdata cfg "${name}" "${trc}"
 }
 
 create_config () {
     configfile=$1
     cleardata cfg
-    cleardata data
 
     setdata data reqlibs ""
 
-    > $CONFH
-    cat <<_HERE_ >> $CONFH
-
+    > ${CONFH}
+    cat <<_HERE_ >> ${CONFH}
 #ifndef __INC_CONFIG_H
 #define __INC_CONFIG_H 1
 
 _HERE_
 
+    if [ -f $CACHEFILE ]; then
+      . $CACHEFILE
+    fi
+    cleardata data          # don't use cache for required libs
+
     cleardata args
     setdata args reqhdr ""
     check_header "_hdr_stdlib" "stdlib.h"
@@ -835,15 +804,17 @@ _HERE_
 
     ininclude=0
     inheaders=1
+    OIFS="$IFS"
     > $INC
-    while read tdatline
-    do
-        if [ $ininclude -eq 1 ]
-        then
-            if [ "${tdatline}" = "endinclude" ]
-            then
+    # This while loop reads data from stdin, so it has
+    # a subshell of its own.  This requires us to save the
+    # configuration data in files for re-use.  See setdata()
+    while read tdatline; do
+        if [ $ininclude -eq 1 ]; then
+            if [ "${tdatline}" = "endinclude" ]; then
                 echo "end include" >> $LOG
                 ininclude=0
+                IFS="$OIFS"
             else
                 echo "${tdatline}" >> $INC
             fi
@@ -856,17 +827,18 @@ _HERE_
                 hdr*|sys*)
                     ;;
                 *)
-                    if [ $inheaders -eq 1 ]
-                    then
+                    if [ $inheaders -eq 1 ]; then
                         check_include_malloc '_include_malloc'
                         check_include_string '_include_string'
+                        check_include_time '_include_time'
                     fi
                     inheaders=0
                     ;;
             esac
         fi
 
-        case ${tdatline} in
+        if [ $ininclude -eq 0 ]; then
+          case ${tdatline} in
             hdr*|sys*)
                 set $tdatline
                 type=$1
@@ -885,6 +857,17 @@ _HERE_
                 setdata args reqhdr "$reqhdr"
                 check_header $nm $hdr
                 ;;
+            const*)
+                set $tdatline
+                name=$2
+                constant=$2
+                shift;shift
+                reqhdr="$*"
+                name=`echo ${name} | tr '[A-Z]' '[a-z]'`
+                name="_const_${name}"
+                setdata args reqhdr "$reqhdr"
+                check_constant $name $constant
+                ;;
             typ*)
                 set $tdatline
                 name=$2
@@ -903,13 +886,11 @@ _HERE_
                 setdata args otherlibs "${libs}"
                 check_lib $nm "${func}"
                 rc=$?
-                if [ $func = 'setmntent' -a $rc -eq 1 ]
-                then
+                if [ $func = 'setmntent' -a $rc -eq 1 ]; then
                     check_setmntent_1arg '_setmntent_1arg'
                     check_setmntent_2arg '_setmntent_2arg'
                 fi
-                if [ $func = 'statfs' -a $rc -eq 1 ]
-                then
+                if [ $func = 'statfs' -a $rc -eq 1 ]; then
                     check_statfs_2arg '_statfs_2arg'
                     check_statfs_3arg '_statfs_3arg'
                     check_statfs_4arg '_statfs_4arg'
@@ -936,14 +917,14 @@ _HERE_
                 func=$2
                 req=$3
                 has=1
-                if [ "${req}" != "" ]
-                then
+                if [ "${req}" != "" ]; then
                     has=`getdata cfg "${req}"`
                 fi
-                if [ ${has} -eq 1 ]
-                then
-                    nm=`echo "_npt_${func}" | tr '[A-Z]' '[a-z]'`
+                nm=`echo "_npt_${func}" | tr '[A-Z]' '[a-z]'`
+                if [ ${has} -eq 1 ]; then
                     check_npt "${nm}" "${func}"
+                else
+                    setdata cfg "${nm}" "0"
                 fi
                 ;;
             dcl*)
@@ -951,11 +932,9 @@ _HERE_
                 type=$2
                 var=$3
                 nm=`echo "_dcl_${var}" | tr '[A-Z]' '[a-z]'`
-                if [ "$type" = "int" ]
-                then
+                if [ "$type" = "int" ]; then
                     check_int_declare $nm $var
-                elif [ "$type" = "ptr" ]
-                then
+                elif [ "$type" = "ptr" ]; then
                     check_ptr_declare $nm $var
                 fi
                 ;;
@@ -976,41 +955,42 @@ _HERE_
             include*)
                 echo "start include" >> $LOG
                 ininclude=1
+                IFS="
+"
                 ;;
-        esac
+          esac
+          savedata
+        fi
     done < ../${configfile}
 
-    . "./cfg${VARFILESUFFIX}"
+    # refetch the configuration data
+    . ${CACHEFILE}
 
-    for cfgvar in ${di_cfg_vars}
-    do
+    for cfgvar in ${di_cfg_vars}; do
         val=`getdata cfg $cfgvar`
-        if [ "${val}" = "0" ]
-        then
-            echo "#undef  ${cfgvar}" >> $CONFH
-        else
-            case ${cfgvar} in
-                _siz_*)
-                    echo "#define ${cfgvar} ${val}" >> $CONFH
-                    ;;
-                *)
-                    echo "#define ${cfgvar} 1" >> $CONFH
-                    ;;
-            esac
+        tval=0
+        if [ "$val" != "0" ]; then
+          tval=1
         fi
+        case ${cfgvar} in
+            _siz_*)
+                echo "#define ${cfgvar} ${val}" >> ${CONFH}
+                ;;
+            *)
+                echo "#define ${cfgvar} ${tval}" >> ${CONFH}
+                ;;
+        esac
     done
 
-    . "./data${VARFILESUFFIX}"
-
     > $REQLIB
-    val=`getdata data reqlibs`
-    val=`for tval in $val
-    do
-        echo $tval
-    done | sort | uniq`
+    val=`getdata data reqlibs`;
+    val=`for tval in $val; do
+            echo $tval
+        done | sort | uniq`
     echo $val >> $REQLIB
 
-    cat << _HERE_ >> $CONFH
+    # standard header for all...
+    cat << _HERE_ >> ${CONFH}
 
 #if ! _key_void || ! _proto_stdc
 # define void int
@@ -1041,26 +1021,87 @@ _HERE_
 
 }
 
+usage () {
+  echo "Usage: $0 [-c <cache-file>] [-o <output-file>]"
+  echo "       [-l <log-file>] [-t <tmp-dir>] [-r <reqlib-file>]"
+  echo "       [-C] <config-file>"
+  echo "  -C : clear cache-file"
+  echo "<tmp-dir> must not exist."
+  echo "defaults:"
+  echo "  <output-file>: config.h"
+  echo "  <cache-file> : mkconfig.cache"
+  echo "  <log-file>   : mkconfig.log"
+  echo "  <tmp-dir>    : _tmp_mkconfig"
+  echo "  <reqlib-file>: reqlibs.txt"
+}
+
 # main
 
-if [ $# -ne 1 ]
-then
-    echo "Usage: $0 <config-file>"
-    exit 1
-fi
-configfile=$1
-echo -n 'test' | grep -- '-n' # > /dev/null 2>&1
+echo -n 'test' | grep -- '-n' > /dev/null 2>&1
 rc=$?
-if [ $rc -eq 0 ]
-then
+if [ $rc -eq 0 ]; then
     EN=''
     EC='\c'
 fi
 
+clearcache=0
+while test $# -gt 1; do
+  case "$1" in
+    -C)
+      shift
+      clearcache=1
+      ;;
+    -c)
+      shift
+      CACHEFILE="$1"
+      shift
+      ;;
+    -o)
+      shift
+      CONFH="$1"
+      shift
+      ;;
+    -l)
+      shift
+      LOG="$1"
+      shift
+      ;;
+    -t)
+      shift
+      TMP="$1"
+      shift
+      ;;
+    -r)
+      shift
+      REQLIB="$1"
+      shift
+      ;;
+  esac
+done
+
+configfile=$1
+if [ $# -ne 1 -o ! -f $configfile ]; then
+  usage
+  exit 1
+fi
+if [ -d $TMP -a $TMP != "_tmp_mkconfig" ]; then
+  usage
+  exit 1
+fi
+
+LOG="../$LOG"
+CONFH="../$CONFH"
+REQLIB="../$REQLIB"
+CACHEFILE="../$CACHEFILE"
+
 test -d $TMP && rm -rf $TMP > /dev/null 2>&1
 mkdir $TMP
 cd $TMP
 
+if [ $clearcache -eq 1 ]; then
+  rm -f $CACHEFILE > /dev/null 2>&1
+fi
+
 echo "$0 using $configfile"
 rm -f $LOG > /dev/null 2>&1
 CFLAGS="${CFLAGS} ${CINCLUDES}"
