@@ -1,7 +1,7 @@
 /*
- * $Id: trimchar.c,v 1.2 2009-11-23 11:24:06-08 bll Exp $
+ * $Id: trimchar.c,v 1.5 2010-01-31 21:07:48-08 bll Exp $
  * $Source: /home/bll/DI/RCS/trimchar.c,v $
- * Copyright 1994-2009 Brad Lanam, Walnut Creek, CA
+ * Copyright 1994-2010 Brad Lanam, Walnut Creek, CA
  */
 
 #include "config.h"
@@ -14,7 +14,7 @@
 #if _hdr_string
 # include <string.h>
 #endif
-#if _hdr_strings && ((! defined (_hdr_string)) || (_include_string))
+#if _hdr_strings && ((! _hdr_string) || (_include_string))
 # include <strings.h>
 #endif
 
@@ -27,7 +27,7 @@ trimChar (str, ch)
     int          ch;
 #endif
 {
-    unsigned int  len;
+    Size_t       len;
 
     len = strlen (str);
     if (len > 0)
